###################################################################
#### Function: Change Settings for wellnames belongig to each plate
###################################################################
## New setting containing vectors with the names of the wells belonging to each well
SETHandleWellnames <- list()

FCollectWellnames <- function(D) {
	ind_start <- 2
	for (i in 1:SETDataLayout$platenumber) {
		ind_end <- (ind_start + SETFilesInput$wells[i] - 1)
		SETHandleWellnames[[paste("plate",i, sep="")]] <<- names(D)[ind_start:ind_end]
		ind_start <- ind_end + 1
	}
	#print(SETHandleWellnames)
}

FCollectWellnames(Data_input)

#########################################################################################
###### Calculations
#########################################################################################

################
## Basic version
################
#######################################
#### Function: Calculating raw data set
#######################################
FCalculationRaw <- function(D) {
	index <- c()
	column <- c(1)
	for (i in 1:length(Template$X)) {
		if (Template$Name[i] != "Exclude") {
			column <- c(column, which(names(D) == Template$X[i]))
			#print(column)
		} else {
			for (j in 1:length(SETHandleWellnames)) {
				index <- which(SETHandleWellnames[[paste("plate",j, sep="")]] == Template$X[i])
				if (length(index) != 0) {
					SETFilesInput$wells[j] <<- SETFilesInput$wells[j] - 1
				}
			}
		}
	}
	
	Data_raw <<- D[,column]
	Data_raw$X <<- (D$X / SETGraphOpt$XScale * SETDataLayout$timescale)
	#print(Data_raw)
	FCollectWellnames(Data_raw)
}

####################################
#### Function: Basic data formatting
####################################
FCalculationFormatting <- function(D, Par = SETAnalysisOpt$FormattingParams) {
	## remove timepoints
	D <- D[which(sort(c(D$X, Par["t1"])) == Par["t1"])[1]:(which(sort(c(D$X, Par["t2"])) == Par["t2"])[1] - 1),]
	## Transform timescale
	D[,1] <- (D[,1] * Par["m"]) + Par["b"]
	## Transform measurement
	for (i in 2:dim(D)[2]) {
		D[,i] <- D[,i] * Par["y"]
	}
	ResFormat <<- D
}

################################
#### Function: Medium Correction
################################
FCalculationMedium <- function(D) {
	
	#Rdfinal will store the data with medium correction, put D into it, in case there are no medium wells on one of the plates
	Rdfinal <- D
	
	#Get numbers of wells with medium
	ind<-c()
    ind<-which(Template$Name == SETAnalysisOpt$name) #Looks for the column numbers that have the correct name in the template.
    wells<-Template$X[ind]    #Converts the ind to the name of the wells that are to be averaged.
	
	#Dt is D temporal, for the time of the loop, refresh Dt
	Dt<-data.frame()
	
	ind_wellfirst <- 2
	
	#If there is more than one plate, loop the medium correction
	for (k in 1:SETDataLayout$platenumber) {
		
		ind_welllast <- (ind_wellfirst + length(SETHandleWellnames[[paste("plate", k, sep = "")]]) -1)
		Dt <- D[,ind_wellfirst:ind_welllast]  #Dt is D temporal, for the time of the loop
		#print(Dt[1:10,])
		
		#Averages the correct values
		#get indices of wells with medium
    	ind2<-c()
		for (i in 1:length(wells)) {
			ind2 <- c(ind2,which(names(Dt) == wells[i]))
		}
		
		#norm_vector_m is a vector with the average of all the normalisation data for each time point.
		norm_vector_m<-c()
		
		if (length(ind2) == 0) {
			ind_wellfirst <- ind_welllast + 1
			next
		} else if (length(ind2) == 1) {
			norm_vector_m <- Dt[,ind2]
		} else if (length(ind2) > 1) {
			norm_vector_m <- rowMeans(Dt[,ind2])
		}
		
		#Set Rd to the right size
		Rd <- Dt
		
		#Recalculate all rows according to medium mean in that row and target value
		for (j in 1:dim(Dt)[1]) {
			Rd[j,] = Dt[j,] / norm_vector_m[j] * SETAnalysisOpt$medTarget
		}  
		
		Rdfinal[,ind_wellfirst:ind_welllast] <- Rd
		
		ind_wellfirst <- ind_welllast + 1
	}
	
	Data_medium <<- Rdfinal
}

############################
#### Function: Mean Template
############################
FTemplateMean <- function() {
	
	fac_names <- as.factor(Template[[SETAnalysisOpt$average]])  #Converts the names of the template to factors.
	fac_names <- levels(fac_names)  #fac_names has all the names of the template only once thanks to level.
	
	Temp <- Template[1:length(fac_names),c(2,4,5)]
	for (i in 1:length(fac_names)) {
		numbers <- which( Template[[SETAnalysisOpt$average]] == fac_names[i])
		Temp[i,] <- c(fac_names[i], as.character(Template$Colour[numbers[1]]), as.character(Template$Linetype[numbers[1]]))
	}
	
	if (length(which(Temp$Name == "Exclude")) != 0) {
		TemplateAverage <<- Temp[-(which(Temp$Name == "Exclude")),]
	} else {
		TemplateAverage <<- Temp
	}
}

###############################
#### Function: Mean Calculation
###############################
FCalculationAverage <- function(D) {
	
	fac_names <- as.factor(Template[[SETAnalysisOpt$average]])  #Converts the names of the template to factors.
	fac_names <- levels(fac_names)  #fac_names has all the names of the template only once thanks to level.
	
	#### Data calculation:
	mat <- matrix(0, nrow = dim(D)[1], ncol = (length(fac_names)+1))
	DAv <- as.data.frame(mat)
	rm(mat)
	names(DAv)[1] <- "X"
	DAv[,1] <- D[,1]
	ind_rem <- c()
	Res <- data.frame()
	
	for (i in 1:length(fac_names)) {
		
		ind <- c()
		
		#get wellnumbers for each name / number / colour
		wells <- Template$X[ which( Template[[SETAnalysisOpt$average]] == fac_names[i] )]
		
		## get indices of wellnumbers in Data set
		for (j in 1:length(wells)) {
			ind <- c(ind, which(names(D) == wells[j]))
		}
		## length is 0 if column was removed (Exclude)
		if (length(ind) == 0) {
			ind_rem <- c(ind_rem, (i+1))
			next
		
		## length is 1 if there is no average to be claculated
		} else if (length(ind) == 1) {
			DAv[,i+1] <- D[, ind]
			names(DAv)[i+1] <- fac_names[i]
		
		## for length > 1 actually calculate the average
		} else {
			mean <- rowMeans(D[, ind])
			DAv[,i+1] <- mean
			names(DAv)[i+1] <- fac_names[i]
		}
	}
	## remove columns that were excluded
	if (length(ind_rem) != 0) DAv <- DAv[,-ind_rem]
	Res <<- DAv
}

############################################
#### Function: Standarddeviation Calculation
############################################
FCalculationStandarddev <- function(D) {
	
	fac_names<-as.factor(Template[[SETAnalysisOpt$average]])  #Converts the names of the template to factors.
	fac_names<-levels(fac_names)  #fac_names has all the names of the template only once thanks to level.
	
	#### Data calculation:
	mat <- matrix(0, nrow = dim(D)[1], ncol = (length(fac_names)+1))
	DSd <- as.data.frame(mat)
	rm(mat)
	names(DSd)[1] <- "X"
	DSd[,1] <- D[,1]
	ind_rem <- c()
	Res <- data.frame()
	
	for (i in 1:length(fac_names)) {
		
		ind <- c()
		
		#get wellnumbers for each name / number / colour
		wells <- Template$X[ which( Template[[SETAnalysisOpt$average]] == fac_names[i] )]
		
		## get indices of wellnumbers in Data set
		for (j in 1:length(wells)) {
			ind <- c(ind, which(names(D) == wells[j]))
		}
		## length is 0 if column was removed (Exclude)
		if (length(ind) == 0) {
			ind_rem <- c(ind_rem, (i+1))
			next
		
		## length is 1 if there is no average to be claculated
		} else if (length(ind) == 1) {
			DSd[,i+1] <- c(rep(0,length(D[, ind])))
			names(DSd)[i+1] <- fac_names[i]
		
		## for length > 1 actually calculate the average
		} else {
			sd <- c()
			for (j in 1:length(D[, ind[1]])) {
				values <- c()
				for (k in 1:length(ind)) {
					values <- c(values, D[j, ind[k]])
				}
				sd <- c(sd, sd(values))
			}
			DSd[,i+1] <- sd
			names(DSd)[i+1] <- fac_names[i]
		}
	}
	## remove columns that were excluded
	if (length(ind_rem) != 0) DSd <- DSd[,-ind_rem]
	Res <<- DSd
}

#####################
#### Function: Smooth
#####################
FCalculationSmooth <- function(D) {
	
	PointsEdge <- (SETAnalysisOpt$smoothParam[2] - 1) / 2
	
	Res <- D
	
	for (i in 2:dim(D)[2]) {
		Temp <- D[,i]
		TempRes <- D[,i]
		for (j in (PointsEdge + 1):(dim(D)[1] - PointsEdge)) {
			TempRes[j] <- mean(Temp[(j - PointsEdge):(j + PointsEdge)])
		}
		Res[,i] <- TempRes
	}
	
	ResSmooth <<- Res[(PointsEdge + 1):(dim(D)[1] - PointsEdge),]
}

##############################
#### Function: Calculate Slope
##############################
FCalculationSlope <- function(D) {
	
	PointsTotal <- dim(D)[1]
	
	## Build data frames
	Dt <- D
	Res <- D
	ResSd <- D
	
	## go through all the lines in the data set to calculate the average values
	for (i in 2:dim(D)[2]) {
		Temp <- D[,i]
		TempRes <- D[,i]
		
		## go through all the points besides the very edge
		for (j in (SlopeEdge + 1):(PointsTotal - SlopeEdge)) {
			TempRes[j] <- mean(Temp[(j - SlopeEdge):(j + SlopeEdge)])
		}
		
		Dt[,i] <- TempRes
	}
	
	ResSmooth <<- Dt[(SlopeEdge + 1):(PointsTotal - SlopeEdge),]
	
	Times <- Dt[,1]
	## go through all the lines to calculate the slope
	for (i in 2:dim(D)[2]) {
		Temp <- Dt[,i]
		TempRes <- Dt[,i]
		TempResSd <- Dt[,i]
		k = SETAnalysisOpt$slopeParam[2]
		## go through all the points within in the fitting border
		for (j in (SlopeBorder + 1):(PointsTotal - SlopeBorder)) {
			fy <- c(Temp[(j - k):(j + k)])
			fx <- c(Times[(j - k):(j + k)])
			fitdata <- data.frame(fx, fy)
			#print(fitdata)
			Fit <- lm(fy ~ fx, fitdata)
			TempRes[j] <- as.numeric(Fit[[1]][2])
			TempResSd[j] <- as.numeric(summary(Fit)$coefficients[2,2])
		}
		Res[,i] <- TempRes
		ResSd[,i] <- TempResSd
	}
	
	ResSlope <<- Res[(SlopeBorder + 1):(PointsTotal - SlopeBorder),]
	ResSlopeSd <<- ResSd[(SlopeBorder + 1):(PointsTotal - SlopeBorder),] 
	#print(ResSlope[1:10])
}

###########################################
#### Function: Calibrate Oxygen Consumption
###########################################
FCalculateCalibration <- function(Oxo, Slope, Parameters) {
	
	numbers <- as.factor(Template$Number)
	numbers <- as.character(levels(numbers))
	
	classes <- list()
	
	Border <- SETAnalysisOpt$slopeParam[2] + ((SETAnalysisOpt$slopeParam[3] - 1) / 2)
	Oxo <- Oxo[(Border + 1):(dim(Oxo)[1]-Border),]
	Oxo <- Oxo[which(sort(c(Oxo$X, Parameters[3])) == Parameters[3])[1]:(which(sort(c(Oxo$X, Parameters[4])) == Parameters[4])[1]-1),]
	Slope <- Slope[which(sort(c(Slope$X, Parameters[3])) == Parameters[3])[1]:(which(sort(c(Slope$X, Parameters[4])) == Parameters[4])[1]-1),]
	
	OxMin <- Parameters[1]
	OxMax <- Parameters[2]

	testdata <- seq(0, 100, length = 301)
	rem <- c()
	
	#names <- c("Ox", as.character(OxMin))
	for(i in 1:length(numbers)) {
		
		## define new class if it doesn't exist yet
		if (length(classes[[as.character(numbers[i])]]) == 0 & numbers[i] != 0) {
			classes[[as.character(numbers[i])]] <- list(	fitdata = data.frame(),
					nlsResult = list(),
					prediction = data.frame(),
					name = ListNamesLayouts[[numbers[i]]][["entry"]]$getText() )
			if (length(classes[[as.character(numbers[i])]]$name) == 0) classes[[as.character(numbers[i])]]$name <- numbers[i]
		} else {
			next
		}
		
		## Get correct data frame
		for (j in 1:length(Template$Number)) {
			if (Template$Number[j] == numbers[i] & Template$Name[j] != "Exclude") {
				dataappend <- data.frame(Oxo[[which(names(Oxo) == Template$X[j])]], Slope[[which(names(Oxo) == Template$X[j])]], row.names = NULL)
				names(dataappend) <- c("Ox","Flux")
				classes[[as.character(numbers[i])]]$fitdata <- rbind(classes[[as.character(numbers[i])]]$fitdata, dataappend)
			}
		}
		
		## Produce fit and prediction
		if (length(which(classes[[as.character(numbers[i])]]$fitdata$Ox >= OxMin & classes[[as.character(numbers[i])]]$fitdata$Ox <= OxMax)) >= 3) {
			classes[[as.character(numbers[i])]]$nlsResult <- lm(	Flux ~ I(as.numeric(ENTCalOxsat$getText()) - Ox) -1,
												data = classes[[as.character(numbers[i])]]$fitdata,
												subset = which(classes[[as.character(numbers[i])]]$fitdata$Ox >= OxMin & classes[[as.character(numbers[i])]]$fitdata$Ox <= OxMax) )
			classes[[as.character(numbers[i])]]$prediction <- predict(classes[[as.character(numbers[i])]]$nlsResult, list(Ox = testdata))
		} else {
			rem <- c(rem, numbers[i])
			next
		}
		classes[[as.character(numbers[i])]]$prediction <- as.data.frame(classes[[as.character(numbers[i])]]$prediction)		
	}
	
	if (length(rem) != 0) {
		for(i in 1: length(rem)) {
			classes <- classes[-which(names(classes) == rem[i])]
		}
	}
	assign("CalibrationResults", classes, pos=-1, envir=.GlobalEnv)
}

###########################################
#### Function: Calculate Oxygen Consumption
###########################################
FCalculationCons <- function(D, DSlope, DSd = data.frame()) {
	
	Border <- SETAnalysisOpt$slopeParam[2] + ((SETAnalysisOpt$slopeParam[3] - 1) / 2)
	PointsTotal <- dim(D)[1]
	
	## Build data frames
	D <- D[(Border + 1):(PointsTotal - Border),]
	ResCons <- D
	ResConsSd <- D
	
	if (dim(DSd)[1] != 0) {
		DSd <- DSd[(Border + 1):(PointsTotal - Border),]
		for(i in 2:dim(D)[2]) {
			ResCons[,i] <- ((SETAnalysisOpt$ConsParam["kLa"] * (SETAnalysisOpt$ConsParam["saturation"] - D[,i])) - DSlope[,i]) * SETAnalysisOpt$ConsParam["conversion"]
			ResConsSd[,i] <- sqrt(((SETAnalysisOpt$ConsParam["saturation"] - D[,i]) * SETAnalysisOpt$ConsParam["conversion"] * SETAnalysisOpt$ConsParam["kLaerror"])^2 + (SETAnalysisOpt$ConsParam["kLa"] * SETAnalysisOpt$ConsParam["conversion"] * DSd[,i])^2)
		}
	} else {
		for(i in 2:dim(D)[2]) {
			ResCons[,i] <- ((SETAnalysisOpt$ConsParam["kLa"] * (SETAnalysisOpt$ConsParam["saturation"] - D[,i])) - DSlope[,i]) * SETAnalysisOpt$ConsParam["conversion"]
			ResConsSd[,i] <- sqrt(((SETAnalysisOpt$ConsParam["saturation"] - D[,i]) * SETAnalysisOpt$ConsParam["conversion"] * SETAnalysisOpt$ConsParam["kLaerror"])^2)
		}
	}
	
	ResCons <<- ResCons
	ResConsSd <<- ResConsSd
}

##############################
#### Function: IC50 Calulation
##############################
FCalculationIC50 <- function() {
	
	ResCoef <- data.frame()
	ResMod <- list()
	
	## reduction of dataset to chosen time points
	if (length(grep("Sm", SETAnalysisOpt$IC50)) == 1) {
		Border <- (SETAnalysisOpt$smoothParam[2] - 1) / 2
		end <- SETAnalysisOpt$IC50Time["pos_end"] - Border
		start <- SETAnalysisOpt$IC50Time["pos_start"] - Border
	} else if (length(grep("Cons", SETAnalysisOpt$IC50)) == 1) {
		end <- SETAnalysisOpt$IC50Time["pos_end"] - SlopeBorder
		start <- SETAnalysisOpt$IC50Time["pos_start"] - SlopeBorder
	} else {
		end <- SETAnalysisOpt$IC50Time["pos_end"]
		start <- SETAnalysisOpt$IC50Time["pos_start"]
	}
	if (end < dim(get(SETAnalysisOpt$IC50))[1]) D <- get(SETAnalysisOpt$IC50)[start:end,]
	else D <- get(SETAnalysisOpt$IC50)[start:dim(get(SETAnalysisOpt$IC50))[1],]
	row.names(D) <- as.numeric(row.names(D)) - start + 1
	rem <- c()
	for (i in 1:dim(D)[1]) {
		if ((i-1) %% SETAnalysisOpt$IC50Time["skip"] != 0) rem <- c(rem, i)
	}
	if (length(rem) != 0) D <-D[-rem,]
		
	## reduce data set to given concentrations
	rem <- c()
	doses <- c()
	for (i in 2:dim(D)[2]) {
		if (length(which(Template$X == names(D)[i])) != 0 ) name <- Template$Name[which(Template$X == names(D)[i])]
		else if (length(which(TemplateAverage$Name == names(D)[i])) != 0) name <- names(D)[i]
		if (SETAnalysisOpt$IC50doses$Dose[which(SETAnalysisOpt$IC50doses$Name == name)] == -1) {
			rem <- c(rem, i)
		} else {
			doses <- c(doses, as.numeric(SETAnalysisOpt$IC50doses$Dose[which(SETAnalysisOpt$IC50doses$Name == name)]))
		}
	}
	
	## Construction of results data set
	# Coefficients
	ResCoef <- D[,1:5]
	names(ResCoef) <- c("X", "IC50", "std.error", "t-value", "p-value")
		
	## Create Calulation Basis data set
	if (length(rem) != 0) D <- D[,-rem]
	D <- rbind(c(-1, doses), D)
	
	Data_IC50Basis <<- D
	
	## Remove time column
	Timepoints <- D[,1]
	D <- D[,-1]
	
	## Get function for fitting
	if (SETAnalysisOpt$IC50model == "LL.2") func <- LL.2()
	else if (SETAnalysisOpt$IC50model == "LL.3") func <- LL.3()
	else if (SETAnalysisOpt$IC50model == "LL.4") func <- LL.4()
	else if (SETAnalysisOpt$IC50model == "LL.5") func <- LL.5()
	
	## Calculate drm for every timepoint and save models
	temp <- data.frame(response = as.numeric(D[2,]), dose = as.numeric(D[1,]))
	for (i in 2:dim(D)[1]) {
		temp$response <- as.numeric(D[i,])
		model <- drm(temp, fct = func)
		ResCoef[(i-1),2:5] <- summary(model)$coefficients[4,]
		ResMod[[as.character(Timepoints[i])]] <- model
	}
	
	Data_IC50Models <<- ResMod
	Data_IC50 <<- ResCoef
	
}

#############
## 24 version
#############
################################
#### Function: OxoDish Correction
################################

					#OD-1133-01  #HD-1109-01  #HD-10-10-01  #First oxygen   #OD-1228-01
#Span=2193			#2161        #13.07       #14.27        #2134           #2193
#Plateau=-209.8		#-201.6      #1.78        #4.446        #-210,8         #-209,8

ln<-function(x){log(x,exp(1))} #log function with exp(1) as a basis

FCalculationOxoDish <- function(D) {
	
	Span <- SETAnalysisOpt$span
	Plateau <- SETAnalysisOpt$plateau
	
	Data_oxoDish <- data.frame()
		
	#Determine which lines to average
	ind_t2 <- SETAnalysisOpt$range[3]
	ind_t1 <- SETAnalysisOpt$range[3] - SETAnalysisOpt$range[1] + 1
	
	# if data was formatted already, reverse the scaling
	if (length(SETAnalysisOpt$Formatting) != 0 && SETAnalysisOpt$Formatting == "Data_raw") {
		for(i in 2:dim(D)[2]) {
			D[,i] <- D[,i] / SETAnalysisOpt$FormattingParams["y"]
		}
		ind_t2 <- which(Data_raw$X == (Data_rawOrig$X[ind_t2] * SETAnalysisOpt$FormattingParams["m"] + SETAnalysisOpt$FormattingParams["b"]))
		ind_t1 <- ind_t2 - SETAnalysisOpt$range[1] + 1
	}
	if (ind_t1 <= 0) {
		ind_t1 <- 1
		print(paste("You removed part of the timepoints you wanted to include in the sensor correction. Only", ind_t2, "timepoints were used.", sep= " "))
	}
	
	#Value_frame stores the n last measurements for each well before they are averaged
	value_frame <- data.frame()
	value_frame <- D[ind_t1:ind_t2, 2:dim(D)[2]]
	
	
	#### New Verison:
	# first logarithmic scale
	value_frameln <- value_frame
	for (i in 1:dim(value_frame)[1]) {
		value_frameln[i,] <- ln( (value_frame[i,] - Plateau)/Span )
	}
	#calculate mean
	norm_vectorln <- c()
	norm_vectorln <- colMeans(value_frameln)
	
	#Clear the data frame Rd.  
	Rd <- D

	#Get the normalisation value by averaging the ln values of each curve in each plate and store it.
	#It is stored the same number of times as the number of curves.
	norm_valueln <- c()
	ind_wellfirst <- 1
	for (i in 1:SETDataLayout$platenumber) { # for each plate
		ind_welllast <- (ind_wellfirst + length(SETHandleWellnames[[paste("plate", i, sep = "")]]) - 1)
		for (j in ind_wellfirst:ind_welllast) {
			norm_valueln[j] <- mean(norm_vectorln[ind_wellfirst:ind_welllast]) #calculate mean for wells belonging to plate
		}
		ind_wellfirst <- ind_welllast + 1
	}
	#print(norm_valueln)
	
	#Recalibrate the data Rd
	for (i in 2:dim(D)[2]) {
		Rd[,i] <- ln( (D[,i] - Plateau) / Span) #Transform the data to ln((Y-Plateau)/Span) (curve to a straight line).
		Rd[,i] <- Rd[,i] / norm_vectorln[i-1] * norm_valueln[i-1] #Divides the data by norm_vectorln and multiplies it by the normalisation value for each plate.
		Rd[,i] <- ( exp(Rd[,i]) * Span ) + Plateau #The data is converted back to exponential
	}

	## Normalisation
	#Value_frame stores the n last measurements for each well AFTER they are averaged
	value_frame <- data.frame()
	value_frame <- Rd[ind_t1:ind_t2, 2:dim(Rd)[2]]
	
	#Norm_vector is a normalisation vector, with the normalisation value for each curve
	norm_vector <- c()
	norm_vector <- colMeans(value_frame) 
	
	#Normalise the data: adjust the corrected values to the real ones
	for (i in 2:dim(D)[2]) {
		Rd[,i] <- Rd[,i]/norm_vector[i-1]*SETAnalysisOpt$oxoDishTarget #Divides the data by norm_value and multiplies it by the incubator value for each plate.
		if (length(SETAnalysisOpt$Formatting) != 0 && SETAnalysisOpt$Formatting == "Data_raw") Rd[,i] <- Rd[,i] * SETAnalysisOpt$FormattingParams["y"]
	}
	
	Data_oxoDish <<- Rd
	#print(Data_oxoDish)
}

#############
## 96 version
#############
##################################
#### Function: OxoPlate Conversion
##################################
FCalculationOxoPlate <- function(D) {
	
	Rdfinal <- D
		
	ind100 <- which(Template$Name == SETAnalysisOpt$cal0100[2]) #Looks for the column numbers that have the correct name in the template.
	wells100 <- Template$X[ind100] #Converts the ind to the name of the wells that are to be averaged.
	
	ind0 <- which(Template$Name == SETAnalysisOpt$cal0100[1]) #Looks for the column numbers that have the correct name in the template.
	wells0 <- Template$X[ind0] #Converts the ind to the name of the wells that are to be averaged.
	
	ind_wellfirst <- 2
	
	# Calibrate every plate
	for (k in 1:SETDataLayout$platenumber) {
		
		ind_welllast <- (ind_wellfirst + length(SETHandleWellnames[[paste("plate", k, sep = "")]]) -1)
		Dt <- D[,ind_wellfirst:ind_welllast]  #Dt is D temporal, for the time of the loop
		#print(Dt[1:10,])
		
		#get indices of wells with cal 100
		ind100 <- c()
		for (i in 1:length(wells100)) {
			ind100 <- c(ind100, which(names(Dt) == wells100[i]))
		}
		#get indices of wells with cal 0
		ind0 <- c()
		for (i in 1:length(wells0)) {
			ind0 <- c(ind0, which(names(Dt) == wells0[i]))
		}
		
		# If one is 0 go to next plate
		if (length(ind0) == 0 | length(ind100) == 0) {
			ind_wellfirst <- ind_welllast + 1
			next
		}
		
		## Get average values
		k100 <- c()
		if (length(ind100) == 1) {
			k100 <- c(Dt[,ind100])
		} else {
			k100 <- c(rowMeans(Dt[,ind100]))
		}
		k0 <- c()
		if (length(ind0) == 1) {
			k0 <- c(Dt[,ind0])
		} else {
			k0 <- c(rowMeans(Dt[,ind0]))
		}
		
		Rd <- Dt
		
		# Calculation of the partial pressure pO2 in % of air saturation
		# pO2=100*(k0/IR-1)/(k0/k100-1)
		
		for (i in 1:dim(Dt)[1]) {
			Rd[i,] <- 100*((k0[i]/Dt[i,])-1)/((k0[i]/k100[i])-1)
		}
		
		Rd <- Rd * SETAnalysisOpt$factor
		
		Rdfinal[,ind_wellfirst:ind_welllast] <- Rd
		
		ind_wellfirst <- ind_welllast + 1
	}
	
	Data_oxoPlate <<- Rdfinal
}

###################################
#### Function: Calibrate HydroPlate
###################################
FCalculationHydroPlate <- function() {
	
	D <- Data_raw
	ResCalHydro <- data.frame()
	RespHHydro <- D
	
	## reduce data set to given concentrations
	rem <- c()
	pHs <- c()
	for (i in 2:dim(D)[2]) {
		if (length(which(Template$X == names(D)[i])) != 0) name <- Template$Name[which(Template$X == names(D)[i])]
		else if (length(which(TemplateAverage$Name == names(D)[i])) != 0) name <- names(D)[i]
		if (SETAnalysisOpt$hydroPlateCal$pH[which(SETAnalysisOpt$hydroPlateCal$Name == name)] == -1) {
			rem <- c(rem, i)
		} else {
			pHs <- c(pHs, as.numeric(SETAnalysisOpt$hydroPlateCal$pH[which(SETAnalysisOpt$hydroPlateCal$Name == name)]))
		}
	}
	
	## Construction of results data set
	# Coefficients
	ResCalHydro <- D[,1:5]
	names(ResCalHydro) <- c("X", "Imin", "Imax", "pH0", "dpH")
		
	## Create Calulation Basis data set
	if (length(rem) != 0) DCal <- D[,-rem]
	else DCal <- D
	DCal <- rbind(c(-1, pHs), DCal)
	
	## Remove time column
	Timepoints <- DCal[,1]
	DCal <- DCal[,-1]
	
	levels_pHs <- levels(as.factor(pHs))
	for (i in 1:length(levels_pHs)) {
		index <- which(DCal[1,] == pHs[i])
		if (length(index) > 2) {
			average <- c(rowMeans(DCal[,index]))
			DCal <- cbind(DCal[-index], average)
		} 
	}
	
	## Get function for fitting
	func <- L.4()
	
	## Calculate drm for every timepoint and save coefficients
	temp <- data.frame(response = as.numeric(DCal[2,]), dose = as.numeric(DCal[1,]))
	for (i in 2:dim(DCal)[1]) {
		temp$response <- as.numeric(DCal[i,])
		model <- drm(temp, fct = func)
		ResCalHydro[(i-1),2:4] <- summary(model)$coefficients[2:4,1]
		ResCalHydro[(i-1),5] <- -1 / (summary(model)$coefficients[1,1])
	}
	
	Data_hydroPlateCal <<- ResCalHydro
	
	pHmin <- min(pHs)
	notificationLowpH <<- c()
	for (i in 1:dim(D)[1]) {
		for (j in 2:dim(D)[2]) {
			RespHHydro[i,j] <- tryCatch(log((ResCalHydro[i, "Imin"] - ResCalHydro[i, "Imax"]) / (D[i,j] - ResCalHydro[i, "Imax"]) -1) * ResCalHydro[i, "dpH"] + ResCalHydro[i, "pH0"], warning = function(W) {
				notificationLowpH <<- c(notificationLowpH, j)
				return(pHmin)
			})
		}
	}
	
	if (length(notificationLowpH) != 0) {
		notificationLowpH <- levels(as.factor(notificationLowpH))
		notification <- names(D)[as.numeric(notificationLowpH[1])]
		if (length(notificationLowpH) > 1) {
			for (i in 2:length(notificationLowpH)) {
				notification <- paste(notification, names(D)[as.numeric(notificationLowpH[i])], sep = ", ")
			}
		}
		print(paste("In the wells", notification, "the pH went lower than the lowest calibration pH, which is", paste0(pHmin, "."), sep=" "))
	}
	
	Data_hydroPlate <<- RespHHydro
}

#########################################################################################
###### Graphs
#########################################################################################

################################################
#### Function: open Drawing area as Cairo device
################################################
FOpenCairo <- function(DA) {
	DrawingAreas <- list()
	DrawingAreas[[paste("DA",DA, sep="")]] <- gtkDrawingArea()
	BXCurrentGraph$packStart(DrawingAreas[[paste("DA",DA, sep="")]], expand = TRUE, fill = TRUE)
	DrawingAreas[[paste("DA",DA, sep="")]]$setSizeRequest(200,200)
	asCairoDevice(DrawingAreas[[paste("DA",DA, sep="")]], pointsize = SETGraphOutput$Scalebar[1])
	Sys.sleep(.1)
	#Background is white
	par(bg = "white")
}

########################################
#### Function: Open Cairo png for Saving
########################################
FSaveCairo <- function(filename, w, h, pt) {
	Cairo(width = w, height = h, pointsize = pt, filename = paste(SETDataOutput$folder, filename, sep="/"))
	Sys.sleep(.1)
}

#### Function: Plotting with legend at the bottom:
##################################################
FPlotStandard <- function(D, DSd, T, subnumber, lines = SETGraphOutput[[LBLCurrentGraphData$getLabel()]], sd = CKBShowSd$getActive(), altYLabel = c()) {
	# Create a palette with all the colors available
	palette(colors())
	
	## Generate proper layout:
	## The layout is mainly incfluenced by the position of the legend.
	## If there should be no legend displayed outside the actual plot area,
	## no layout specification is required. Otherwise the layout is dependant
	## on the number of rows (legend below the plot) and columns (legend beside the plot)
	## to display in the legend.
	# Legend below the plot:
	if (SETGraphOutput$Scalebar["position"] == 1) {
		rows <- length(T$Name)/SETGraphOutput$Scalebar["columns"]
		switch (as.character(ceiling(rows)),
		"1" = layout(matrix(c(rep(1,30),rep(2,5)),7,5, byrow=TRUE)),
		"2" = layout(matrix(c(rep(1,25),rep(2,5)),6,5, byrow=TRUE)),
		layout(matrix(c(rep(1,20),rep(2,5)),5,5, byrow=TRUE)) )
	# Legend to the right of the plot:
	} else if (SETGraphOutput$Scalebar["position"] == 2) {
		switch(as.character(SETGraphOutput$Scalebar["columns"]),
		"1" = layout(matrix(c(rep(1,6),2,rep(1,6),2,rep(1,6),2,rep(1,6),2,rep(1,6),2),5,7, byrow=TRUE)),
		"2" = layout(matrix(c(rep(1,5),2,rep(1,5),2,rep(1,5),2,rep(1,5),2,rep(1,5),2),5,6, byrow=TRUE)),
		layout(matrix(c(rep(1,4),2,rep(1,4),2,rep(1,4),2,rep(1,4),2,rep(1,4),2),5,5, byrow=TRUE)) )
	}
	
	## Define white space around the plot depending on layout and
	## whitespace options. Altered parameters are mar (margins), tcl
	## (tick length), mgp (axis positions), xaxs / yaxs (axis length).
	VARParMin <- list( mar = c(1.9, 1.8, 0.1, 0.1),
	                   mgp = c(0.8, 0, 0),
					   tcl = 0.5,
					   xaxs = "e",
					   yaxs = "e" )
	VARParNew <- VARParMin
	VARParNew$mar[1:2] <- rep(VARParMin$mar[1] + 3.3 * SETGraphOutput$Scalebar["whitespace"], 2)
	VARParNew$mar[3:4] <- rep(VARParMin$mar[4] + 1.8 * SETGraphOutput$Scalebar["whitespace"], 2)
	VARParNew$mgp[1] <- VARParMin$mgp[1] + 2.2 * SETGraphOutput$Scalebar["whitespace"]
	VARParNew$mgp[2] <- VARParMin$mgp[2] + 1 * SETGraphOutput$Scalebar["whitespace"]
	if (SETGraphOutput$Scalebar["whitespace"] > 0.5) {
		VARParNew$tcl <- -0.5
		VARParNew$xaxs <- "r"
		VARParNew$yaxs <- "r"
	} else {
		VARParNew$tcl <- 0.5
		VARParNew$xaxs <- "i"
		VARParNew$yaxs <- "i"
	}
	if (SETGraphOpt$TitleShow == TRUE) VARParNew$mar[3] <- VARParNew$mar[3] + 1 + 1 * SETGraphOutput$Scalebar["whitespace"]
	if (SETGraphOpt$SubsShow == TRUE) VARParNew$mar[3] <- VARParNew$mar[3] + 1 + 1 * SETGraphOutput$Scalebar["whitespace"]
	par(VARParNew)
	
	if (length(altYLabel) == 0) yLabel <- SETGraphOpt$YLabel
	else yLabel <- altYLabel
	
	#Set x and y limits
	if (SETGraphOpt$XLimits[1] == 10000) {
		SETGraphOpt$XLimits[1] <- min(D[,1])
	}
	if (SETGraphOpt$XLimits[2] == 10000) {
		SETGraphOpt$XLimits[2] <- max(D[,1])
	}
	if (SETGraphOpt$YLimits[1] == 10000) {
		SETGraphOpt$YLimits[1] <- min(D[,2:dim(D)[2]])
	}
	if (SETGraphOpt$YLimits[2] == 10000) {
		SETGraphOpt$YLimits[2] <- max(D[,2:dim(D)[2]])
	}
	
	plot(	D$X,
			D$X,
			col = "white",
			type = "l",
			xlim = SETGraphOpt$XLimits,
			ylim = SETGraphOpt$YLimits,
			xlab = SETGraphOpt$XLabel,
			ylab = yLabel,
			axes = FALSE )
	
	if (SETGraphOpt$TitleShow == TRUE && SETGraphOpt$SubsShow == TRUE) {
		title(main = SETGraphOpt$Title, line = 1 + 1.8 * SETGraphOutput$Scalebar["whitespace"])
		mtext(SETGraphOpt$Subtitles[[subnumber]], line = 0.1 + SETGraphOutput$Scalebar["whitespace"], cex = 0.9)
	} else if (SETGraphOpt$TitleShow == TRUE && SETGraphOpt$SubsShow == FALSE) {
		title(main = SETGraphOpt$Title, line = 0.2 + 1.3 * SETGraphOutput$Scalebar["whitespace"])
	} else if (SETGraphOpt$TitleShow == FALSE && SETGraphOpt$SubsShow == TRUE) {
		mtext(SETGraphOpt$Subtitles[[subnumber]], line = 0.1 + 1.3 * SETGraphOutput$Scalebar["whitespace"], cex = 0.9)
	}
	
	## Add Standarddeviation ranges
	errorcol = paste0("grey", (100 - SETGraphOutput$Scalebar["error"]))
	if (sd == TRUE) {
		xx <- c(D$X, rev(D$X))
		for (i in 1:length(lines)) {
			if (lines[i] == TRUE) {
				lband <- D[,(i+1)] - DSd[,(i+1)]
				uband <- D[,(i+1)] + DSd[,(i+1)]
				yy <- c(lband, rev(uband))
				polygon(xx, yy, col = errorcol, border = errorcol)
			}
		}
	}
	
	gridcol = paste0("grey", (100 - SETGraphOutput$Scalebar["grid"]))	
	if (gridcol != "grey100" && SETGraphOutput$Scalebar["lwd"] >= 0.8) grid(col = gridcol, lwd = (3/4*SETGraphOutput$Scalebar["lwd"]))
	else if (gridcol != "grey100") grid(col = gridcol, lwd = 0.5)
	
	## Add lines
	for (i in 1:length(lines)) {
		if (lines[i] == TRUE) {
			lines(	D$X,
					D[,(i+1)],
					col = as.character(T$Colour[i]),
					lty = as.character(T$Linetype[i]),
					lwd = SETGraphOutput$Scalebar["lwd"] )
			if (dim(D)[1] == 1) {
				points(	D$X,
						D[,(i+1)],
						col = as.character(T$Colour[which(T$X == names(D)[i+1])]),
						type = "p")
			}
		}
	}
	
	if (SETGraphOutput$Scalebar["lwd"] >= 0.8) {
		box(lwd = (3/4*SETGraphOutput$Scalebar["lwd"]))
		axis(1, col = "transparent", col.ticks = "black", col.axis = "black", lwd = (3/4*SETGraphOutput$Scalebar["lwd"]))
		axis(2, col = "transparent", col.ticks = "black", col.axis = "black", lwd = (3/4*SETGraphOutput$Scalebar["lwd"]))
	} else {
		box(lwd = 0.5)
		axis(1, col = "transparent", col.ticks = "black", col.axis = "black", lwd = 0.5)
		axis(2, col = "transparent", col.ticks = "black", col.axis = "black", lwd = 0.5)
	}
	
	if (SETGraphOutput$Scalebar["position"] == 1) par(mar=c(VARParNew$mar[c(4,2)], 0.1, VARParNew$mar[4]))
	else if (SETGraphOutput$Scalebar["position"] == 2) par(mar=c(VARParNew$mar[1], 0.1, VARParNew$mar[3:4]))
	if (SETGraphOutput$Scalebar["position"] != 0) {
		if (length(T$X) == 0) {
			legendcolour <- c()
			linetype <- c()
			names <- c()
			number <- c()
			for (i in 1:length(lines)) {
				if (lines[i] == TRUE) {
					legendcolour <- c(legendcolour, as.character(T$Colour[i]))
					linetype <- c(linetype, as.character(T$Linetype[i]))
					names <- c(names, as.character(T$Name[i]))
					number <- c(number, min(as.numeric(Template$Number[which(Template$Name == T$Name[i])])))
				}
			}
		} else {
			legendcolour <- c()
			linetype <- c()
			names <- c()
			number <- c()
			for (i in 1:length(lines)) {
				if (lines[i] == TRUE) {
					legendcolour <- c(legendcolour, as.character(T$Colour[i]))
					linetype <- c(linetype, as.character(T$Linetype[i]))
					names <- c(names, as.character(T$X[i]))
					number <- c(number, T$Number)
				}
			}
		}
		
		sortnum <- sort(as.numeric(number))
		sortname <- c()
		sortline <- c()
		sortcol <- c()
		
		if (length(unique(number)) == length(number)) {
			for(i in 1:length(sortnum)) {
				sortname[i] <- names[which(number == sortnum[i])]
				sortcol[i] <- legendcolour[which(number == sortnum[i])]
				sortline[i] <- linetype[which(number == sortnum[i])]
			}
		} else {
			sortname <- names
			sortline <- linetype
			sortcol <- legendcolour
		}
		
		if (SETGraphOutput$Scalebar["position"] == 1 || SETGraphOutput$Scalebar["position"] == 2) {
			plot(	0,
					col=1,
					xlab="",
					ylab="",
					axes=FALSE )
		
			if (length(names) != 0) {
				legend(	"center",
						legend = sortname,
						ncol = SETGraphOutput$Scalebar["columns"],
						col = sortcol,
						lty = sortline,
						lwd = SETGraphOutput$Scalebar["lwd"],
						bty = "n",
						seg.len = 2.3 ) #seg.len = 1.5
			}
			if (SETGraphOutput$Scalebar["lwd"] >= 0.8) box(lwd = (3/4*SETGraphOutput$Scalebar["lwd"]))
			else box(lwd = 0.5)
		} else {
			position <- switch(as.character(SETGraphOutput$Scalebar["position"]),
			        "3" = "topleft", "4" = "top", "5" = "topright",
					"6" = "right",
					"7" = "bottomright", "8" = "bottom", "9" = "bottomleft",
					"10" = "left" )
			if (length(names) != 0) {
				legend(	position,
						legend = sortname,
						ncol = SETGraphOutput$Scalebar["columns"],
						col = sortcol,
						lty = sortline,
						lwd = SETGraphOutput$Scalebar["lwd"],
						bty = "o",
						seg.len = 2.3) #seg.len = 1.5
			}
		}
	}
}

#### Function: Plotting IC50 timeresolved
#########################################
FPlotIC50Time <- function(D, sd = CKBShowSd$getActive()) {
	# Create a palette with all the colors available
	palette(colors())
	
	## For the IC50 timeresolved plot no legend is needed and thus 
	## only the white space around the plot needs to be defined.
	## Altered parameters are mar (margins), tcl
	## (tick length), mgp (axis positions), xaxs / yaxs (axis length).
	VARParMin <- list( mar = c(1.9, 1.8, 0.1, 0.1),
	                   mgp = c(0.8, 0, 0),
					   tcl = 0.5,
					   xaxs = "e",
					   yaxs = "e" )
	VARParNew <- VARParMin
	VARParNew$mar[1:2] <- rep(VARParMin$mar[1] + 3.3 * SETGraphOutput$Scalebar["whitespace"], 2)
	VARParNew$mar[3:4] <- rep(VARParMin$mar[4] + 1.8 * SETGraphOutput$Scalebar["whitespace"], 2)
	VARParNew$mgp[1] <- VARParMin$mgp[1] + 2.2 * SETGraphOutput$Scalebar["whitespace"]
	VARParNew$mgp[2] <- VARParMin$mgp[2] + 1 * SETGraphOutput$Scalebar["whitespace"]
	if (SETGraphOutput$Scalebar["whitespace"] > 0.5) {
		VARParNew$tcl <- -0.5
		VARParNew$xaxs <- "r"
		VARParNew$yaxs <- "r"
	} else {
		VARParNew$tcl <- 0.5
		VARParNew$xaxs <- "i"
		VARParNew$yaxs <- "i"
	}
	if (SETGraphOpt$TitleShow == TRUE) VARParNew$mar[3] <- VARParNew$mar[3] + 1 + 1 * SETGraphOutput$Scalebar["whitespace"]
	if (SETGraphOpt$SubsShow == TRUE) VARParNew$mar[3] <- VARParNew$mar[3] + 1 + 1 * SETGraphOutput$Scalebar["whitespace"]
	par(VARParNew)
	
	#Set x and y limits
	if (SETGraphOpt$XLimits[1] == 10000) {
		SETGraphOpt$XLimits[1] <- min(D[,1])
	}
	if (SETGraphOpt$XLimits[2] == 10000) {
		SETGraphOpt$XLimits[2] <- max(D[,1])
	}
	if (SETGraphOpt$YLimits[1] == 10000) {
		SETGraphOpt$YLimits[1] <- min(D[,2])
	}
	if (SETGraphOpt$YLimits[2] == 10000) {
		SETGraphOpt$YLimits[2] <- max(D[,2])
	}
	
	plot(	D$X,
			D$X,
			col = "white",
			type = "l",
			xlim = SETGraphOpt$XLimits,
			ylim = SETGraphOpt$YLimits,
			xlab = SETGraphOpt$XLabel,
			ylab = SETAnalysisOpt$IC50Axis,
			axes = TRUE )
	
	if (SETGraphOpt$TitleShow == TRUE && SETGraphOpt$SubsShow == TRUE) {
		title(main = SETGraphOpt$Title, line = 1 + 1.8 * SETGraphOutput$Scalebar["whitespace"])
		mtext(SETGraphOpt$Subtitles[["IC50Time"]], line = 0.1 + SETGraphOutput$Scalebar["whitespace"], cex = 0.9)
	} else if (SETGraphOpt$TitleShow == TRUE && SETGraphOpt$SubsShow == FALSE) {
		title(main = SETGraphOpt$Title, line = 0.2 + 1.3 * SETGraphOutput$Scalebar["whitespace"])
	} else if (SETGraphOpt$TitleShow == FALSE && SETGraphOpt$SubsShow == TRUE) {
		mtext(SETGraphOpt$Subtitles[["IC50Time"]], line = 0.1 + 1.3 * SETGraphOutput$Scalebar["whitespace"], cex = 0.9)
	}
	
	## Add Standarddeviation ranges
	errorcol = paste0("grey", (100 - SETGraphOutput$Scalebar["error"]))
	if (sd == TRUE) {
		xx <- c(D$X, rev(D$X))
		lband <- D$IC50 - D[["std.error"]]
		uband <- D$IC50 + D[["std.error"]]
		yy <- c(lband, rev(uband))
		for (i in 2:(length(yy)-1)) {
			if (is.nan(yy[i]) == TRUE) {
				yylow <- mean(D[,3])
				for (j in i:1) {
					if (is.nan(yy[j]) == FALSE) {
						yylow <- yy[j]
						break
					}
				}
				yytop <- mean(D[,3])
				for (j in i:length(yy)) {
					if (is.nan(yy[j]) == FALSE) {
						yytop <- yy[j]
						break
					}
				}
				yy[i] <- mean(yylow, yytop)
			}
		}
		polygon(xx, yy, col = errorcol, border = errorcol)
	}
	
	gridcol = paste0("grey", (100 - SETGraphOutput$Scalebar["grid"]))
	if (gridcol != "grey100") grid(col = gridcol)
	
	## Add lines
	lines(	D$X,
			D$IC50,
			col = "black",
			lty = "solid",
			lwd = SETGraphOutput$Scalebar[3])
	
	box()
}

#### Function: Plotting IC50 fitting data
#########################################
FPlotIC50Fit <- function(D, lines = SETGraphOutput[[LBLCurrentGraphData$getLabel()]]) {
	#Create a palette with all the colors available
	# Create a palette with all the colors available
	palette(colors())
	
	## Generate proper layout:
	## The layout is mainly incfluenced by the position of the legend.
	## If there should be no legend displayed outside the actual plot area,
	## no layout specification is required. Otherwise the layout is dependant
	## on the number of rows (legend below the plot) and columns (legend beside the plot)
	## to display in the legend.
	# Legend below the plot:
	if (SETGraphOutput$Scalebar["position"] == 1) {
		rows <- length(lines[lines == TRUE])/SETGraphOutput$Scalebar["columns"]
		switch (as.character(ceiling(rows)),
		"1" = layout(matrix(c(rep(1,30),rep(2,5)),7,5, byrow=TRUE)),
		"2" = layout(matrix(c(rep(1,25),rep(2,5)),6,5, byrow=TRUE)),
		layout(matrix(c(rep(1,20),rep(2,5)),5,5, byrow=TRUE)) )
	# Legend to the right of the plot:
	} else if (SETGraphOutput$Scalebar["position"] == 2) {
		switch(as.character(SETGraphOutput$Scalebar["columns"]),
		"1" = layout(matrix(c(rep(1,6),2,rep(1,6),2,rep(1,6),2,rep(1,6),2,rep(1,6),2),5,7, byrow=TRUE)),
		"2" = layout(matrix(c(rep(1,5),2,rep(1,5),2,rep(1,5),2,rep(1,5),2,rep(1,5),2),5,6, byrow=TRUE)),
		layout(matrix(c(rep(1,4),2,rep(1,4),2,rep(1,4),2,rep(1,4),2,rep(1,4),2),5,5, byrow=TRUE)) )
	}
	
	## Define white space around the plot depending on layout and
	## whitespace options. Altered parameters are mar (margins), tcl
	## (tick length), mgp (axis positions), xaxs / yaxs (axis length).
	VARParMin <- list( mar = c(1.9, 1.8, 0.1, 0.1),
	                   mgp = c(0.8, 0, 0),
					   tcl = 0.5,
					   xaxs = "e",
					   yaxs = "e" )
	VARParNew <- VARParMin
	VARParNew$mar[1:2] <- rep(VARParMin$mar[1] + 3.3 * SETGraphOutput$Scalebar["whitespace"], 2)
	VARParNew$mar[3:4] <- rep(VARParMin$mar[4] + 1.8 * SETGraphOutput$Scalebar["whitespace"], 2)
	VARParNew$mgp[1] <- VARParMin$mgp[1] + 2.2 * SETGraphOutput$Scalebar["whitespace"]
	VARParNew$mgp[2] <- VARParMin$mgp[2] + 1 * SETGraphOutput$Scalebar["whitespace"]
	if (SETGraphOutput$Scalebar["whitespace"] > 0.5) {
		VARParNew$tcl <- -0.5
		VARParNew$xaxs <- "r"
		VARParNew$yaxs <- "r"
	} else {
		VARParNew$tcl <- 0.5
		VARParNew$xaxs <- "i"
		VARParNew$yaxs <- "i"
	}
	if (SETGraphOpt$TitleShow == TRUE) VARParNew$mar[3] <- VARParNew$mar[3] + 1 + 1 * SETGraphOutput$Scalebar["whitespace"]
	if (SETGraphOpt$SubsShow == TRUE) VARParNew$mar[3] <- VARParNew$mar[3] + 1 + 1 * SETGraphOutput$Scalebar["whitespace"]
	par(VARParNew)
	
	#Set y limits
	if (SETGraphOpt$YLimits[1] == 10000) {
		SETGraphOpt$YLimits[1] <- min(Data_IC50Basis[2:dim(Data_IC50Basis)[1], 2:dim(Data_IC50Basis)[2]])
	}
	if (SETGraphOpt$YLimits[2] == 10000) {
		SETGraphOpt$YLimits[2] <- max(Data_IC50Basis[2:dim(Data_IC50Basis)[1], 2:dim(Data_IC50Basis)[2]])
	}
	
	plot(	D[[1]],
			col = "white",
			type = as.character(SETGraphOutput$IC50["type"]),
			ylim = SETGraphOpt$YLimits,
			xlab = SETAnalysisOpt$IC50Axis,
			ylab = SETGraphOpt$YLabel )
	
	gridcol = paste0("grey", (100 - SETGraphOutput$Scalebar["grid"]))
	if (gridcol != "grey100") grid(col = gridcol)
	
	if (SETGraphOpt$TitleShow == TRUE && SETGraphOpt$SubsShow == TRUE) {
		title(main = SETGraphOpt$Title, line = 1 + 1.8 * SETGraphOutput$Scalebar["whitespace"])
		mtext(SETGraphOpt$Subtitles[["IC50Fits"]], line = 0.1 + SETGraphOutput$Scalebar["whitespace"], cex = 0.9)
	} else if (SETGraphOpt$TitleShow == TRUE && SETGraphOpt$SubsShow == FALSE) {
		title(main = SETGraphOpt$Title, line = 0.2 + 1.3 * SETGraphOutput$Scalebar["whitespace"])
	} else if (SETGraphOpt$TitleShow == FALSE && SETGraphOpt$SubsShow == TRUE) {
		mtext(SETGraphOpt$Subtitles[["IC50Fits"]], line = 0.1 + 1.3 * SETGraphOutput$Scalebar["whitespace"], cex = 0.9)
	}
	
	legendcolour <- c()
	linetype <- c()
	names <- c()
		
	if (SETGraphOutput$IC50["diff"] == "colour") {
		for (i in 1:length(lines)) {
			if (lines[i] == TRUE) {
				plot(	D[[i]],
						col = (i%%100) + 20,
						lty = "solid",
						type = as.character(SETGraphOutput$IC50["type"]),
						lwd = SETGraphOutput$Scalebar[3],
						add = TRUE )
				names <- c(names, as.character(round(as.numeric(names(D)[i]), 3)))
				legendcolour <- c(legendcolour, colors()[(i%%100)+20])
				linetype <- c(linetype, "solid")
			}
		}
	}
	
	else if (SETGraphOutput$IC50["diff"] == "lty") {
		for (i in 1:length(lines)) {
			if (lines[i] == TRUE) {
				plot(	D[[i]],
						col = "black",
						lty = c(1,2,3,4,5,6)[((i*1777)%%5)+1],
						pch = i%%18,
						type = as.character(SETGraphOutput$IC50["type"]),
						lwd = SETGraphOutput$Scalebar[3],
						add = TRUE )
				names <- c(names, as.character(round(as.numeric(names(D)[i]), 3)))
				legendcolour <- c(legendcolour, "black")
				linetype <- c(linetype, c(1,2,3,4,5,6)[((i*1777)%%5)+1])
			}
		}
	}
	
	else if (SETGraphOutput$IC50["diff"] == "none") {
		for (i in 1:length(lines)) {
			if (lines[i] == TRUE) {
				plot(	D[[i]],
						col = "black",
						lty = "solid",
						type = as.character(SETGraphOutput$IC50["type"]),
						lwd = SETGraphOutput$Scalebar[3],
						add = TRUE )
				names <- c(names, as.character(round(as.numeric(names(D)[i]), 3)))
				legendcolour <- c(legendcolour, "black")
				linetype <- c(linetype, "solid")
			}
		}
	}
		
	box()
	
	if (SETGraphOutput$Scalebar["position"] == 1) par(mar=c(VARParNew$mar[c(4,2)], 0.1, VARParNew$mar[4]))
	else if (SETGraphOutput$Scalebar["position"] == 2) par(mar=c(VARParNew$mar[1], 0.1, VARParNew$mar[3:4]))
	if (SETGraphOutput$Scalebar["position"] == 1 || SETGraphOutput$Scalebar["position"] == 2) {
		plot(	0,
				col=1,
				xlab="",
				ylab="",
				axes=FALSE )
	
		if (length(names) != 0) {
			legend(	"center",
					legend = names,
					col = legendcolour,
					lty = linetype,
					lwd = 2,
					bty = "n",
					cex = 0.8,
					ncol = SETGraphOutput$Scalebar[2],
					seg.len = 2.3 ) #seg.len = 1.5
		}
		box()
	} else if (SETGraphOutput$Scalebar["position"] != 0) {
		position <- switch(as.character(SETGraphOutput$Scalebar["position"]),
			        "3" = "topleft", "4" = "top", "5" = "topright",
					"6" = "right",
					"7" = "bottomright", "8" = "bottom", "9" = "bottomleft",
					"10" = "left" )
	
		if (length(names) != 0) {
			legend(	position,
					legend = names,
					col = legendcolour,
					lty = linetype,
					lwd = 2,
					bty = "o",
					cex = 0.8,
					ncol = SETGraphOutput$Scalebar[2],
					seg.len = 2.3 ) #seg.len = 1.5
		}
	}
}