#########################################################################################
####  Acquired Settings
#########################################################################################
## In this section all parameters and selections required for the analysis are acquired.
## They are all saved in the list SETAnalysisOpt:
## - average: character string giving grouping criterion for average calculation
## - SD: logical value whether to calculate standard deviation or not
## - Formatting: character naming the data set to format
## - FormattingParams: vector of five numerical parameters for reformatting the data:
## - - t1 = time start cut
## - - t2 = time end cut
## - - m = time scale factor
## - - b = time scale shift
## - - y = measurement scale factor
## - smoothParam: vector of two parameters for smoothing:
## - - calc = logical whether to smooth data or not
## - - range = odd number defining the neighbourhood to smooth
## - smoothSelection: named character vector of variable length containing logical value for each data set
## - slopeParam: vector of three parameters for slope calculation:
## - - calc = logical whether to calculate slope or not
## - - points = integer for how many points to include on each side
## - - average = odd number defining the neighbourhood to smooth before calculating the slope
## - slopeSelection: named character vector of variable length containing logical value for each data set
## - raw: vector of two logicals; first is always TRUE, second is whether to calculate average of raw data
## - oxoDish: vector of two logicals; first is whether to do OxoDish sensor correction, second is whether to calculate average of corrected data
## - range: vector of three parameters defining the data basis for sensor correction:
## - - number = number of points to use
## - - last = exact timepoint of last value to use
## - - indlast = calculated index of last value to use
## - oxoDishTarget: numerical value defining target concentration of the sensor correction
## - span: lot specific numerical parameter
## - plateau: lot specific numerical parameter
## - oxoPlate: vector of two logicals; first is whether to convert OxoPlate data, second is whether to calculate average of converted data
## - cal0100: vector of two character strings; template names of 0 % oxygen wells (first) and 100 % oxygen wells (second)
## - unitselections: vector of three parameters each indicating the selected index of a combobox
## - - unit = unit to convert to
## - - temperature = measurement condition
## - - pressure = measurement condition
## - factor: numerical value giving the unit conversion factor for OxoPlate conversion
## - medium: vector of two logicals; first is whether to calculate (medium) normalisation, second is whether to calculate average of normalised data
## - name: template name of condition to normalise to
## - medTarget: target concentration for (medium) normalisation
## - hydroPlate: vector of two logicals; first is whether to calibrate a HydroPlate, second is whether to calculate average of calibrated data
## - hydroPlateCal: data frame containing all template names and corresponding pH values for HydroPlate calibration
## - IC50calc: logical value whether to calculate time resolved IC50
## - IC50: character string naming the data basis for IC50 calculation
## - IC50Time: vector containing five numerical parameters to define the time points of IC50 calculation:
## - - time_start = exact first time point
## - - pos_start = calculated index of first time point
## - - time_end = exact last time point
## - - pos_end = calculated index of last time point
## - - skip = IC50 is only calculated for every nth time point, n = skip
## - IC50model: character naming the function to use as model for IC50 calculation
## - IC50doses: data frame containing containing all template names and substance concentrations for IC50 calculation
## - IC50Axis: character giving alternative x-axis label for IC50 graphs
## - ConsParam: vector containing six parameters needed for the conversion to oxygen consumption:
## - - calc = logical whether to do conversion or not
## - - saturation = saturated oxygen concentration
## - - kLa = oxygen transfer coefficient fitting the experimental setup
## - - conversion = unit conversion factor applied after calculation
## - - kLaerror = error of the kLa for calculation of total conversion error
## - - load = logical whether values were loaded from a file
## - ConsCharacters: vector of three character strings for oxygen conversion:
## - - file = file name  / path to load parameters
## - - calibration = name of the calibration selected from the file
## - - axis = alternative y-axis label for consumed oxygen graphics
## - ConsSelection: named character vector of variable length containing logical value for each data set

SETAnalysisOpt <- list(	average = "name",
						SD = TRUE,
						Formatting = "",
						FormattingParams = c("t1" = 0, "t2" = 1, "m" = 1, "b" = 0, "y" = 1),
						smoothParam = c(calc = TRUE, range = 3),
						smoothSelection = c(),
						slopeParam = c(calc = TRUE, points = 3, average = 3),
						slopeSelection = c(),
						raw = c(TRUE, FALSE),
						oxoDish = c(FALSE, FALSE),
						range = c("number" = 10, "last" = 11, "indlast" = 0),
						oxoDishTarget = 95,
						span = 2000,
						plateau = -200,
						oxoPlate = c(FALSE, FALSE),
						cal0100 = c("Cal0", "Cal100"),
						unitselections = c("unit" = 0, "temperature" = 0, "pressure" = 0),
						factor = 0.209,
						medium = c(TRUE, TRUE),
						name = "Medium",
						medTarget = 80,
						hydroPlate = c(FALSE, FALSE),
						hydroPlateCal = data.frame(),
						IC50calc = TRUE,
						IC50 = "Data_raw",
						IC50Time = c("time_start" = 0, "pos_start" = 1, "time_end" = 111, "pos_end" = 10, "skip" = 5),
						IC50model = "LL.3",
						IC50doses = data.frame(),
						IC50Axis = "dose",
						ConsParam = c("calc" = TRUE, "saturation" = 95, "kLa" = 8, "conversion" = 0.211, "kLaerror" = 1, "load" = FALSE),
						ConsCharacters = c("file" = "none", "calibration" = "none", "axis" = "consumed oxygen"),
						ConsSelection = c() )

#########################################################################################
#### Layout and Interface Definition for Analysis Options
#########################################################################################

####
#### Frame Layout
#################
## The interface for the analysis options is separated into two main parts. At the top
## the selection of the different analysis steps to carry out is always displayed,
## together with some utility buttons at the right border of the screen. In the lower
## part of the screen a notebook is displayed, which holds a tab for each selected
## analysis step in order to acquire all the necessary parameters.

LBLAnalysisOptionsFrame <- gtkLabel("Please select the calculations you want to run and the corresponding options.")
LBLAnalysisOptionsFrame$setSizeRequest(-1, 25)

# Top selection frame:
FRSelection <- gtkFrame("Analysis selection")

# Notebook tabs:
FRMediumCal <- gtkFrame()
FRAverage <- gtkFrame()
FRSmooth <- gtkFrame()
FRSlope <- gtkFrame()
FRCons <- gtkFrame()
FRIC50 <- gtkFrame()
FRFormat <- gtkFrame()
# Notebook assembly:
NBAnalysisOptions <- gtkNotebook()
NBAnalysisOptions["tab-hborder"] <- 10
LBLPageFormat <- gtkLabel("Basic data formatting")
NBAnalysisOptions$appendPage(FRFormat, tab.label = LBLPageFormat)
LBLPageAverage <- gtkLabel("Average")
NBAnalysisOptions$appendPage(FRAverage, tab.label = LBLPageAverage)
LBLPageSmooth <- gtkLabel("Data smoothing")
NBAnalysisOptions$appendPage(FRSmooth, tab.label = LBLPageSmooth)
LBLPageSlope <- gtkLabel("Numerical slope")
NBAnalysisOptions$appendPage(FRSlope, tab.label = LBLPageSlope)
LBLPageCons <- gtkLabel("Oxygen consumption")
NBAnalysisOptions$appendPage(FRCons, tab.label = LBLPageCons)
LBLPageMedium <- gtkLabel("Normalisation")
NBAnalysisOptions$appendPage(FRMediumCal, tab.label = LBLPageMedium)
LBLPageIC50 <- gtkLabel("IC50")
NBAnalysisOptions$appendPage(FRIC50, tab.label = LBLPageIC50)

# Analysis Options assembly:
layoutAnalysis <- gtkVBox(FALSE, 5)
	layoutAnalysis$packStart(LBLAnalysisOptionsFrame, expand = FALSE, fill = FALSE)
	layoutAnalysis$packStart(FRSelection, expand = FALSE, fill = FALSE)
	layoutAnalysis$packStart(NBAnalysisOptions, expand = TRUE, fill = TRUE)

####
#### Selection Frame
####################
## In the main selection frame the user can select the analysis steps.
## It is divided horizontally and first comes the selection of the basic
## steps. In the base version these are only the raw data acquiring and
## the normalisation to one condition (currently called (medium) normalisation).
## Each basic calculation (besides raw data) has two checkboxes - one for
## doing the analysis step itself and one for calculating the average afterwards.
#### Next in the horizontal line comes an arrow indicating that after
## these basic analysis steps the process can be extended by several other
## calculations.

BXSelection <- gtkHBox(FALSE, 10)
BXSelection$setBorderWidth(5)
FRSelection$setBorderWidth(3)
	FRSelection$add(BXSelection)

BXSelectionBase <- gtkVBox(FALSE, 5)
BXSelectionArrow <- gtkHBox()
ARRFurtherAnalysis <- gtkArrow(arrow.type="GTK_ARROW_RIGHT")
ARRFurtherAnalysis$setSizeRequest(70,20)
ARRFurtherAnalysis$modifyFg("GTK_STATE_NORMAL", "grey60")
	BXSelectionArrow$packStart(ARRFurtherAnalysis)
BXSelectionExtended <- gtkVBox(FALSE, 5)
BXAnalysisButtons <- gtkVBox(FALSE, 5)
	BXSelection$packStart(BXSelectionBase, expand = FALSE, fill = FALSE)
	BXSelection$packStart(BXSelectionArrow, expand = FALSE, fill = FALSE)
	BXSelection$packStart(BXSelectionExtended, expand = FALSE, fill = TRUE)
	BXSelection$packEnd(BXAnalysisButtons, expand = FALSE, fill = FALSE)

# ## Basic calculations:
BXSelectionBaseLabel <- gtkHBox(FALSE, 5)
SEPSelectionBaseLabel <- gtkLabel("")
BXSelectionRaw <- gtkHBox(FALSE, 5)
SEPSelection3 <- gtkHSeparator()
BXSelectionMedium <- gtkHBox(FALSE, 5)
	BXSelectionBase$packStart(BXSelectionBaseLabel, expand = FALSE, fill = FALSE)
	BXSelectionBase$packStart(SEPSelectionBaseLabel, expand = FALSE, fill = FALSE)
	BXSelectionBase$packStart(BXSelectionRaw, expand = FALSE, fill = FALSE)
# Label
LBLSelectionBase <- gtkLabel("Basic calculations:")
	BXSelectionBaseLabel$packStart(LBLSelectionBase, expand = TRUE, fill = FALSE)
# Raw
LBLSelectionRaw <- gtkLabel("Raw data")
LBLSelectionRaw$setSizeRequest(160, -1)
CKBSelectionRawAv <- gtkCheckButton("calculate average")
CKBSelectionRawAv$setSizeRequest(150, -1)
	BXSelectionRaw$packStart(LBLSelectionRaw, expand = FALSE, fill = FALSE)
	BXSelectionRaw$packStart(CKBSelectionRawAv, expand = FALSE, fill = FALSE)
# Normalisation
CKBSelectionMedium <- gtkCheckButton("Normalisation")
CKBSelectionMedium$setSizeRequest(160, -1)
CKBSelectionMediumAv <- gtkCheckButton("calculate average")
CKBSelectionMediumAv$setSizeRequest(150, -1)
	BXSelectionMedium$packStart(CKBSelectionMedium, expand = FALSE, fill = FALSE)
	BXSelectionMedium$packStart(CKBSelectionMediumAv, expand = FALSE, fill = FALSE)

#### These extended calculations are data smoothing, numerical slope calculation,
## conversion to oxygen consumption and calculation of time resolved IC50 data.
## They are each displayed with a simple checkbox.
# ## Extended calculations:
BXSelectionExtendedLabel <- gtkHBox(FALSE, 5)
SEPSelectionExtendedLabel <- gtkLabel("")
BXSelectionExtendedTop <- gtkHBox(FALSE, 5)
SEPSelectionExtended <- gtkHSeparator()
BXSelectionExtendedBottom <- gtkHBox(FALSE, 5)
BXSelectionSmooth <- gtkHBox(FALSE, 5)
BXSelectionSmooth$setSizeRequest(150,-1)
BXSelectionSlope <- gtkHBox(FALSE, 5)
BXSelectionCons <- gtkHBox(FALSE, 5)
BXSelectionCons$setSizeRequest(150,-1)
BXSelectionIC50 <- gtkHBox(FALSE, 5)
	BXSelectionExtended$packStart(BXSelectionExtendedLabel, expand = FALSE, fill = FALSE)
	BXSelectionExtended$packStart(SEPSelectionExtendedLabel, expand = FALSE, fill = FALSE)
	BXSelectionExtended$packStart(BXSelectionExtendedTop, expand = FALSE, fill = FALSE)
	BXSelectionExtended$packStart(SEPSelectionExtended, expand = FALSE, fill = FALSE)
	BXSelectionExtended$packStart(BXSelectionExtendedBottom, expand = FALSE, fill = FALSE)
		BXSelectionExtendedTop$packStart(BXSelectionSmooth, expand = FALSE, fill = FALSE)
		BXSelectionExtendedTop$packStart(BXSelectionSlope, expand = FALSE, fill = FALSE)
		BXSelectionExtendedBottom$packStart(BXSelectionCons, expand = FALSE, fill = FALSE)
		BXSelectionExtendedBottom$packStart(BXSelectionIC50, expand = FALSE, fill = FALSE)
# Label
LBLSelectionExtended <- gtkLabel("Also calculate:")
	BXSelectionExtendedLabel$packStart(LBLSelectionExtended, expand = TRUE, fill = FALSE)
# Smooth
CKBSelectionSmooth <- gtkCheckButton("data smoothing")
	BXSelectionSmooth$packStart(CKBSelectionSmooth, expand = FALSE, fill = FALSE)
# Slope
CKBSelectionSlope <- gtkCheckButton("slope")
	BXSelectionSlope$packStart(CKBSelectionSlope, expand = FALSE, fill = FALSE)
# Consumption
CKBSelectionCons <- gtkCheckButton("consumed oxygen")
	BXSelectionCons$packStart(CKBSelectionCons, expand = FALSE, fill = FALSE)
# IC50
CKBSelectionIC50 <- gtkCheckButton("IC50")
	BXSelectionIC50$packStart(CKBSelectionIC50, expand = FALSE, fill = FALSE)

#### To the right border of the top section three buttons are displayed, which
## trigger some useful functions. The first button prints the time points of the
## raw data set in its original as well as its converted scale. This allows the user
## to remain in the R-Environment when looking for certain time points to type in
## parameters. The second button prints the dimensions of all calculated data sets.
## This is a useful tool when users want to further process the data in R or
## to get the number of data points retrieved from the IC50 calculation. The last
## button is for calibration of the parameters of the oxygen consumption. It opens
## a popup which allows to do this particular step entirely separated from
## the rest of the analysis process.
# ## Buttons
BXAnalysisButtonsLabel <- gtkHBox(FALSE, 5)
BXAnalysisButtonsbox <- gtkVButtonBox(TRUE)
	BXAnalysisButtons$packStart(BXAnalysisButtonsLabel, expand = FALSE, fill = FALSE)
	BXAnalysisButtons$packStart(BXAnalysisButtonsbox, expand = FALSE, fill = FALSE)
# Label
LBLAnalysisButtons <- gtkLabel("Additional functions:")
	BXAnalysisButtonsLabel$packStart(LBLAnalysisButtons, expand = FALSE, fill = FALSE)
# Buttons
btnPrintTimepoints <- gtkButton("Print timepoints")
btnPrintDimensions <- gtkButton("Print dataset dimensions")
btnSwitchCalibration <- gtkButton("Calibrate oxygen consumption")
	BXAnalysisButtonsbox$packStart(btnPrintTimepoints)
	BXAnalysisButtonsbox$packStart(btnPrintDimensions)
	BXAnalysisButtonsbox$packStart(btnSwitchCalibration)

####
#### Basic formatting frame
###########################

BXFormat <- gtkVBox(FALSE, 5)
BXFormat$setBorderWidth(5)
FRFormat$setBorderWidth(3)
	FRFormat$add(BXFormat)

BXDataLayoutTimeLabel <- gtkHBox(FALSE, 5)
BXDataLayoutTime <- gtkHBox(FALSE, 5)
BXPlotAxesH1Label <- gtkHBox(FALSE, 5)
BXPlotAxesH1 <- gtkHBox(FALSE, 5) #Time Scale
SEPFormat1 <- gtkLabel("")
SEPFormat2 <- gtkHSeparator()
SEPFormat3 <- gtkLabel("")
BXFormatExplX <- gtkHBox(FALSE, 5)
SEPFormat4 <- gtkLabel("")
BXFormatSelectData <- gtkHBox(FALSE, 5)
SEPFormat5 <- gtkLabel("")
BXFormatCutX <- gtkHBox(FALSE, 5)
BXFormatTransformX <- gtkHBox(FALSE, 5)
SEPFormat6 <- gtkLabel("")
BXFormatTransformY <- gtkHBox(FALSE, 5)
BXFormatRerun <- gtkHBox(FALSE, 5)
	BXFormat$packStart(BXDataLayoutTimeLabel, expand = FALSE, fill = FALSE)
	BXFormat$packStart(BXDataLayoutTime, expand = FALSE, fill = FALSE)
	BXFormat$packStart(BXPlotAxesH1Label, expand = FALSE, fill = FALSE)
	BXFormat$packStart(BXPlotAxesH1, expand = FALSE, fill = FALSE)
	BXFormat$packStart(SEPFormat1, expand = FALSE, fill = FALSE)
	BXFormat$packStart(SEPFormat2, expand = FALSE, fill = FALSE)
	BXFormat$packStart(SEPFormat3, expand = FALSE, fill = FALSE)
	BXFormat$packStart(BXFormatExplX, expand = FALSE, fill = FALSE)
	BXFormat$packStart(SEPFormat4, expand = FALSE, fill = FALSE)
	BXFormat$packStart(BXFormatSelectData, expand = FALSE, fill = FALSE)
	BXFormat$packStart(SEPFormat5, expand = FALSE, fill = FALSE)
	BXFormat$packStart(BXFormatCutX, expand = FALSE, fill = FALSE)
	BXFormat$packStart(BXFormatTransformX, expand = FALSE, fill = FALSE)
	BXFormat$packStart(SEPFormat6, expand = FALSE, fill = FALSE)
	BXFormat$packStart(BXFormatTransformY, expand = FALSE, fill = FALSE)
	BXFormat$packEnd(BXFormatRerun, expand = FALSE, fill = FALSE)

## Data Layout time unit
LBLDataLayoutTime <- gtkLabel("What is the input time unit of your measurement?")
	BXDataLayoutTimeLabel$packStart(LBLDataLayoutTime, expand = FALSE, fill = FALSE)
	
btnDataLayoutTimesec <- gtkRadioButton(group = NULL, "sec")
btnDataLayoutTimemin <- gtkRadioButton(group = c(btnDataLayoutTimesec), "min")
btnDataLayoutTimeh <- gtkRadioButton(group = c(btnDataLayoutTimesec), "hour")
btnDataLayoutTimeday <- gtkRadioButton(group = c(btnDataLayoutTimesec), "day")
	BXDataLayoutTime$packStart(btnDataLayoutTimesec, expand = FALSE, fill = FALSE)
	BXDataLayoutTime$packStart(btnDataLayoutTimemin, expand = FALSE, fill = FALSE)
	BXDataLayoutTime$packStart(btnDataLayoutTimeh, expand = FALSE, fill = FALSE)
	BXDataLayoutTime$packStart(btnDataLayoutTimeday, expand = FALSE, fill = FALSE)

## Axes H1
LBLPlotAxesH1 <- gtkLabel("Which should be the output unit of the timescale. If you want a timescale different from the ones displayed, please use the options below.")
	BXPlotAxesH1Label$packStart(LBLPlotAxesH1, expand = FALSE, fill = FALSE)

btnPlotAxesH1sec <- gtkRadioButton(group = NULL, "sec")
btnPlotAxesH1min <- gtkRadioButton(group = c(btnPlotAxesH1sec), "min")
btnPlotAxesH1h <- gtkRadioButton(group = c(btnPlotAxesH1sec), "hour")
btnPlotAxesH1day <- gtkRadioButton(group = c(btnPlotAxesH1sec), "day")
	BXPlotAxesH1$packStart(btnPlotAxesH1sec, expand = FALSE, fill = FALSE)
	BXPlotAxesH1$packStart(btnPlotAxesH1min, expand = FALSE, fill = FALSE)
	BXPlotAxesH1$packStart(btnPlotAxesH1h, expand = FALSE, fill = FALSE)
	BXPlotAxesH1$packStart(btnPlotAxesH1day, expand = FALSE, fill = FALSE)

## Data selection
LBLFormatExplX <- gtkLabel("Here you can do basic formatting of your datasets, including data reduction, timescale transformation and unit conversion.")
LBLFormatSelectData1 <- gtkLabel("Reformat the timescale and the measurement unit of the ")
LBLFormatSelectData2 <- gtkLabel(" . (This will also affect following calculations.)")
COMFormatSelectData <- gtkComboBoxNewText()
COMFormatSelectData$appendText("no formatting")
COMFormatSelectData$setActive(0)
	BXFormatExplX$packStart(LBLFormatExplX, expand = FALSE, fill = FALSE)
	BXFormatSelectData$packStart(LBLFormatSelectData1, expand = FALSE, fill = FALSE)
	BXFormatSelectData$packStart(COMFormatSelectData, expand = FALSE, fill = FALSE)
	BXFormatSelectData$packStart(LBLFormatSelectData2, expand = FALSE, fill = FALSE)

## X Axis formatting
LBLFormatCutX1 <- gtkLabel("Remove data before timepoint t1 = ")
LBLFormatCutX2 <- gtkLabel(" and after t2 = ")
LBLFormatCutX3 <- gtkLabel(" (in given output unit).")
ENTFormatCutXStart <- gtkEntry()
ENTFormatCutXStart$setSizeRequest(50, 20)
ENTFormatCutXEnd <- gtkEntry()
ENTFormatCutXEnd$setSizeRequest(50, 20)
	BXFormatCutX$packStart(LBLFormatCutX1, expand = FALSE, fill = FALSE)
	BXFormatCutX$packStart(ENTFormatCutXStart, expand = FALSE, fill = FALSE)
	BXFormatCutX$packStart(LBLFormatCutX2, expand = FALSE, fill = FALSE)
	BXFormatCutX$packStart(ENTFormatCutXEnd, expand = FALSE, fill = FALSE)
	BXFormatCutX$packStart(LBLFormatCutX3, expand = FALSE, fill = FALSE)
LBLFormatTransformX1 <- gtkLabel("Transform the timescale using the formular t' = m * t + b, with m = ")
LBLFormatTransformX2 <- gtkLabel(" and b = ")
LBLFormatTransformX3 <- gtkLabel(" (applied on data in given output unit).")
ENTFormatTransformXm <- gtkEntry()
ENTFormatTransformXm$setSizeRequest(50, 20)
ENTFormatTransformXb <- gtkEntry()
ENTFormatTransformXb$setSizeRequest(50, 20)
	BXFormatTransformX$packStart(LBLFormatTransformX1, expand = FALSE, fill = FALSE)
	BXFormatTransformX$packStart(ENTFormatTransformXm, expand = FALSE, fill = FALSE)
	BXFormatTransformX$packStart(LBLFormatTransformX2, expand = FALSE, fill = FALSE)
	BXFormatTransformX$packStart(ENTFormatTransformXb, expand = FALSE, fill = FALSE)
	BXFormatTransformX$packStart(LBLFormatTransformX3, expand = FALSE, fill = FALSE)

####
#### Y Axis formatting
######################
LBLFormatTransformY <- gtkLabel("Use the following factor for the conversion of the measurement unit:")
ENTFormatTransformY <- gtkEntry()
ENTFormatTransformY$setSizeRequest(50, 20)
	BXFormatTransformY$packStart(LBLFormatTransformY, expand = FALSE, fill = FALSE)
	BXFormatTransformY$packStart(ENTFormatTransformY, expand = FALSE, fill = FALSE)

## Rerun
btnFormattingRerun <- gtkButton("Rerun basic data formatting only")
btnFormattingRerun$setSizeRequest(250, 30)
btnFormattingRerun["visible"] <- FALSE
	BXFormatRerun$packStart(btnFormattingRerun, expand = TRUE, fill = FALSE)

####
#### Medium Cal Frame
#####################

BXMediumCal <- gtkVBox(FALSE, 5)
BXMediumCal$setBorderWidth(5)
FRMediumCal$setBorderWidth(3)
	FRMediumCal$add(BXMediumCal)

BXMediumCalName <- gtkHBox(FALSE, 5)
BXMediumCalTarget <- gtkHBox(FALSE, 5)
BXMediumCalRerun <- gtkHBox(FALSE, 5)
BXMediumCalRerun$setBorderWidth(5)
	BXMediumCal$packStart(BXMediumCalName, expand = FALSE, fill = FALSE)
	BXMediumCal$packStart(BXMediumCalTarget, expand = FALSE, fill = FALSE)
	BXMediumCal$packEnd(BXMediumCalRerun, expand = FALSE, fill = FALSE)

## Name
LBLMediumCalName <- gtkLabel("Please enter the name of the wells used as a reference for normalisation.")
ENTMediumCalName <- gtkEntry()
ENTMediumCalName$setSizeRequest(-1, 20)
	BXMediumCalName$packStart(LBLMediumCalName, expand = FALSE, fill = FALSE)
	BXMediumCalName$packStart(ENTMediumCalName, expand = TRUE, fill = TRUE)

## Target
LBLMediumCalTarget <- gtkLabel("What is the target value for the normalisation (after optional rescaling of the data)?")
ENTMediumCalTarget <- gtkEntry()
ENTMediumCalTarget$setSizeRequest(50, 20)
	BXMediumCalTarget$packStart(LBLMediumCalTarget, expand = FALSE, fill = FALSE)
	BXMediumCalTarget$packStart(ENTMediumCalTarget, expand = FALSE, fill = FALSE)

## Rerun
btnMediumCalRerun <- gtkButton("Rerun normalisation only")
btnMediumCalRerun$setSizeRequest(250, 30)
btnMediumCalRerun["visible"] <- FALSE
	BXMediumCalRerun$packStart(btnMediumCalRerun, expand = TRUE, fill = FALSE)

####
#### Average Frame
##################

BXAverage <- gtkVBox(FALSE, 5)
BXAverage$setBorderWidth(5)
FRAverage$setBorderWidth(3)
	FRAverage$add(BXAverage)

BXAverageLabel <- gtkHBox(FALSE, 5)
BXAverageMode <- gtkHBox(FALSE, 5)
BXAverageSD <- gtkHBox(FALSE, 5)
BXAverageRerun <- gtkHBox(FALSE, 5)
BXAverageRerun$setBorderWidth(5)
	BXAverage$packStart(BXAverageLabel, expand = FALSE, fill = FALSE)
	BXAverage$packStart(BXAverageMode, expand = FALSE, fill = FALSE)
	BXAverage$packStart(BXAverageSD, expand = FALSE, fill = FALSE)
	BXAverage$packEnd(BXAverageRerun, expand = FALSE, fill = FALSE)

## Label
LBLAverageLabel <- gtkLabel("Please select according to which attribute the average should be calculated.")
	BXAverageLabel$packStart(LBLAverageLabel, expand = FALSE, fill = FALSE)

## Mode
btnAverageName <- gtkRadioButton(group = NULL, "name")
btnAverageNumber <- gtkRadioButton(group = c(btnAverageName), "number")
btnAverageColour <- gtkRadioButton(group = c(btnAverageName), "colour")
	BXAverageMode$packStart(btnAverageName, expand = FALSE, fill = FALSE)
	BXAverageMode$packStart(btnAverageNumber, expand = FALSE, fill = FALSE)
	BXAverageMode$packStart(btnAverageColour, expand = FALSE, fill = FALSE)

## SD
CKBAverageSD <- gtkCheckButton("calculate standard deviation")
	BXAverageSD$packStart(CKBAverageSD, expand = FALSE, fill = FALSE)

## Rerun
btnAverageRerun <- gtkButton("Rerun average/standard deviation only")
btnAverageRerun$setSizeRequest(250, 30)
btnAverageRerun["visible"] <- FALSE
	BXAverageRerun$packStart(btnAverageRerun, expand = TRUE, fill = FALSE)

####
#### Smooth Frame
#################

BXSmooth <- gtkVBox(FALSE, 5)
BXSmooth$setBorderWidth(5)
FRSmooth$setBorderWidth(3)
	FRSmooth$add(BXSmooth)

BXSmoothOption <- gtkHBox(FALSE, 5)
BXSmoothLabel <- gtkHBox(FALSE, 5)
BXSmoothRaw <- gtkHBox(FALSE, 5)
BXSmoothRawAv <- gtkHBox(FALSE, 5)
BXSmoothMedium <- gtkHBox(FALSE, 5)
BXSmoothMediumAv <- gtkHBox(FALSE, 5)
BXSmoothRerun <- gtkHBox(FALSE, 5)
BXSmoothRerun$setBorderWidth(5)
	BXSmooth$packStart(BXSmoothOption, expand = FALSE, fill = FALSE)
	BXSmooth$packStart(BXSmoothLabel, expand = FALSE, fill = FALSE)
	BXSmooth$packStart(BXSmoothRaw, expand = FALSE, fill = FALSE)
	BXSmooth$packStart(BXSmoothRawAv, expand = FALSE, fill = FALSE)
	BXSmooth$packEnd(BXSmoothRerun, expand = FALSE, fill = FALSE)

## Option
LBLSmoothOption1 <- gtkLabel("Smooth the selected datasets by averaging a neighbourhood of")
LBLSmoothOption2 <- gtkLabel("points. (Needs to be an odd number!)")
ENTSmoothOption <- gtkEntry()
	ENTSmoothOption$setSizeRequest(50, 20)
	BXSmoothOption$packStart(LBLSmoothOption1, expand = FALSE, fill = FALSE)
	BXSmoothOption$packStart(ENTSmoothOption, expand = FALSE, fill = FALSE)
	BXSmoothOption$packStart(LBLSmoothOption2, expand = FALSE, fill = FALSE)

## Label
LBLSmoothSel <- gtkLabel("Calculate smoothing for:")
	BXSmoothLabel$packStart(LBLSmoothSel, expand = FALSE, fill = FALSE)

## Checkboxes
CKBSmRaw <- gtkCheckButton("raw data")
	BXSmoothRaw$packStart(CKBSmRaw, expand = FALSE, fill = FALSE)
CKBSmRawAv <- gtkCheckButton("averaged raw data")
	BXSmoothRawAv$packStart(CKBSmRawAv, expand = FALSE, fill = FALSE)
CKBSmMedium <- gtkCheckButton("normalisation data")
	BXSmoothMedium$packStart(CKBSmMedium, expand = FALSE, fill = FALSE)
CKBSmMediumAv <- gtkCheckButton("averaged normalisation data")
	BXSmoothMediumAv$packStart(CKBSmMediumAv, expand = FALSE, fill = FALSE)

## Rerun
btnSmoothRerun <- gtkButton("Rerun smoothing only")
btnSmoothRerun$setSizeRequest(250, 30)
btnSmoothRerun["visible"] <- FALSE
	BXSmoothRerun$packStart(btnSmoothRerun, expand = TRUE, fill = FALSE)

####
#### Numerical slope Frame
##########################

BXSlope <- gtkVBox(FALSE, 5)
BXSlope$setBorderWidth(5)
FRSlope$setBorderWidth(3)
	FRSlope$add(BXSlope)

BXSlopeDisplay <- gtkHBox(FALSE, 5)
BXSlopeOptionSmooth1 <- gtkHBox(FALSE, 5)
BXSlopeOptionSmooth2 <- gtkHBox(FALSE, 5)
BXSlopeOptionPoints <- gtkHBox(FALSE, 5)
BXSlopeLabel <- gtkHBox(FALSE, 5)
BXSlopeRaw <- gtkHBox(FALSE, 5)
BXSlopeRawAv <- gtkHBox(FALSE, 5)
BXSlopeMedium <- gtkHBox(FALSE, 5)
BXSlopeMediumAv <- gtkHBox(FALSE, 5)
BXSlopeRerun <- gtkHBox(FALSE, 5)
BXSlopeRerun <- gtkHBox(FALSE, 5)
BXSlopeRerun$setBorderWidth(5)
	BXSlope$packStart(BXSlopeDisplay, expand = FALSE, fill = FALSE)
	BXSlope$packStart(BXSlopeOptionSmooth1, expand = FALSE, fill = FALSE)
	BXSlope$packStart(BXSlopeOptionSmooth2, expand = FALSE, fill = FALSE)
	BXSlope$packStart(BXSlopeOptionPoints, expand = FALSE, fill = FALSE)
	BXSlope$packStart(BXSlopeLabel, expand = FALSE, fill = FALSE)
	BXSlope$packStart(BXSlopeRaw, expand = FALSE, fill = FALSE)
	BXSlope$packStart(BXSlopeRawAv, expand = FALSE, fill = FALSE)
	BXSlope$packEnd(BXSlopeRerun, expand = FALSE, fill = FALSE)

## Display
LBLSlopeDisplay <- gtkLabel("The unit of the slope will be converted measurement unit / target time unit! When exporting graphs remember to change the Y-axis label accordingly!")
	BXSlopeDisplay$packStart(LBLSlopeDisplay, expand = FALSE, fill = FALSE)

## Options
LBLSlopeSmooth1 <- gtkLabel("Smooth the curves prior to calculation by averaging a neighbourhood of")
LBLSlopeSmooth2 <- gtkLabel("points (needs to be an odd number).")
ENTSlopeSmooth <- gtkEntry()
ENTSlopeSmooth$setSizeRequest(50, 20)
	BXSlopeOptionSmooth1$packStart(LBLSlopeSmooth1, expand = FALSE, fill = FALSE)
	BXSlopeOptionSmooth1$packStart(ENTSlopeSmooth, expand = FALSE, fill = FALSE)
	BXSlopeOptionSmooth1$packStart(LBLSlopeSmooth2, expand = FALSE, fill = FALSE)

LBLSlopePoints1 <- gtkLabel("Use")
LBLSlopePoints2 <- gtkLabel("(smoothed) points on either side of the centre for a linear fit to determine the slope.")
ENTSlopePoints <- gtkEntry()
ENTSlopePoints$setSizeRequest(50, 20)
	BXSlopeOptionPoints$packStart(LBLSlopePoints1, expand = FALSE, fill = FALSE)
	BXSlopeOptionPoints$packStart(ENTSlopePoints, expand = FALSE, fill = FALSE)
	BXSlopeOptionPoints$packStart(LBLSlopePoints2, expand = FALSE, fill = FALSE)

## Label
LBLSlopeSel <- gtkLabel("Calculate slope for:")
	BXSlopeLabel$packStart(LBLSlopeSel, expand = FALSE, fill = FALSE)

## Checkboxes
CKBSlopeRaw <- gtkCheckButton("raw data")
	BXSlopeRaw$packStart(CKBSlopeRaw, expand = FALSE, fill = FALSE)
CKBSlopeRawAv <- gtkCheckButton("averaged raw data")
	BXSlopeRawAv$packStart(CKBSlopeRawAv, expand = FALSE, fill = FALSE)
CKBSlopeMedium <- gtkCheckButton("normalisation data")
	BXSlopeMedium$packStart(CKBSlopeMedium, expand = FALSE, fill = FALSE)
CKBSlopeMediumAv <- gtkCheckButton("averaged normalisation data")
	BXSlopeMediumAv$packStart(CKBSlopeMediumAv, expand = FALSE, fill = FALSE)

## Rerun
btnSlopeRerun <- gtkButton("Rerun slope calculation only")
btnSlopeRerun$setSizeRequest(250, 30)
btnSlopeRerun["visible"] <- FALSE
	BXSlopeRerun$packStart(btnSlopeRerun, expand = TRUE, fill = FALSE)

####
#### Consumption Frame
######################

BXCons <- gtkVBox(FALSE, 5)
BXCons$setBorderWidth(5)
FRCons$setBorderWidth(3)
	FRCons$add(BXCons)

BXConsSat <- gtkHBox(FALSE, 5)
BXConsLoadCal <- gtkHBox(FALSE, 5)
BXConsSelectCal <- gtkHBox(FALSE, 15)
BXConsFactors <- gtkHBox(FALSE, 5)
BXConsAxis <- gtkHBox(FALSE, 5)
BXConsLabel <- gtkHBox(FALSE, 5)
BXConsRaw <- gtkHBox(FALSE, 5)
BXConsRawAv <- gtkHBox(FALSE, 5)
BXConsMedium <- gtkHBox(FALSE, 5)
BXConsMediumAv <- gtkHBox(FALSE, 5)
BXConsRerun <- gtkHBox(FALSE, 5)
BXConsRerun$setBorderWidth(5)
	BXCons$packStart(BXConsSat, expand = FALSE, fill = FALSE)
	BXCons$packStart(BXConsLoadCal, expand = FALSE, fill = FALSE)
	BXCons$packStart(BXConsSelectCal, expand = FALSE, fill = FALSE)
	BXCons$packStart(BXConsFactors, expand = FALSE, fill = FALSE)
	BXCons$packStart(BXConsAxis, expand = FALSE, fill = FALSE)
	BXCons$packStart(BXConsLabel, expand = FALSE, fill = FALSE)
	BXCons$packStart(BXConsRaw, expand = FALSE, fill = FALSE)
	BXCons$packStart(BXConsRawAv, expand = FALSE, fill = FALSE)
	BXCons$packEnd(BXConsRerun, expand = FALSE, fill = FALSE)

## Sat
LBLConsSat1 <- gtkLabel("The fully saturated oxygen concentration is")
LBLConsSat2 <- gtkLabel(".")
ENTConsSat <- gtkEntry()
ENTConsSat$setSizeRequest(70,20)
btnConsSatMedium <- gtkButton("use normalisation target value")
	BXConsSat$packStart(LBLConsSat1, expand = FALSE, fill = FALSE)
	BXConsSat$packStart(ENTConsSat, expand = FALSE, fill = FALSE)
	BXConsSat$packStart(LBLConsSat2, expand = FALSE, fill = FALSE)

## Load Calibration
CKBConsLoadCal <- gtkCheckButtonNewWithLabel("Load calibration data from file:")
ENTConsLoadCal <- gtkEntry()
ENTConsLoadCal$setSizeRequest(100, 20)
btnConsLoadCal <- gtkButton()
IMDirectoryCal <- gtkImageNewFromPixbuf(PBDirectory[[1]])
btnConsLoadCal$setImage(IMDirectoryCal)
	BXConsLoadCal$packStart(CKBConsLoadCal, expand = FALSE, fill = FALSE)
	BXConsLoadCal$packStart(ENTConsLoadCal, expand = TRUE, fill = TRUE)
	BXConsLoadCal$packStart(btnConsLoadCal, expand = FALSE, fill = FALSE)

#### Produce list of available kla values from loaded file
btnKLAValueChoicesGroup <- gtkRadioButtonNew(group = NULL)
btnKLAValueChoicesGroup["visible"] <- FALSE
	BXConsSelectCal$packStart(btnKLAValueChoicesGroup, expand = FALSE, fill = FALSE)
	
ListKLAValueChoices <<- list()
ListKLAValueHandlers <<- list()
FProduceKLAValueChoice <- function() {
	children <- BXConsSelectCal$getChildren()
	if (length(children) > 1) {
		for (i in 2:length(children)) children[[i]]$destroy()
	}
	ListKLAValueChoices <<- list()
	ListKLAValueHandlers <<- list()
	for (i in 1:length(CalibrationResults)) {
		if (class(CalibrationResults[[i]]) == "list" && dim(summary(CalibrationResults[[i]]$nlsResult)$coefficients)[1] != 0) {
			ListKLAValueChoices[[i]] <<- gtkRadioButtonNewWithLabel(label = CalibrationResults[[i]]$name)
			BXConsSelectCal$packStart(ListKLAValueChoices[[i]], expand = FALSE, fill = FALSE)
			ListKLAValueChoices[[i]]$setGroup(btnKLAValueChoicesGroup$getGroup())
			ListKLAValueHandlers[[i]] <<- gSignalConnect(ListKLAValueChoices[[i]], "toggled", after = TRUE, quote({
				for (i in 1:length(ListKLAValueChoices)) {
					if (ListKLAValueChoices[[i]]$getActive() == TRUE) {
						ENTConsFactorsKLA$setText(summary(CalibrationResults[[i]]$nlsResult)$coefficients[1,1])
						SETAnalysisOpt$ConsCharacters["calibration"] <- CalibrationResults[[i]]$name
					}
				}
			}))
		}
	}
	gSignalEmit(ListKLAValueChoices[[1]], "toggled")
}

## Factors
LBLConsFactors1 <- gtkLabel("A kla value of")
LBLConsFactors2 <- gtkLabel("with an error of")
LBLConsFactors3 <- gtkLabel("and a unit conversion factor of")
LBLConsFactors4 <- gtkLabel("will be used for the analysis.")
ENTConsFactorsKLA <- gtkEntry()
ENTConsFactorsKLA$setSizeRequest(70, 20)
ENTConsFactorsKLAError <- gtkEntry()
ENTConsFactorsKLAError$setSizeRequest(70, 20)
ENTConsFactorsUCon <- gtkEntry()
ENTConsFactorsUCon$setSizeRequest(70, 20)
	BXConsFactors$packStart(LBLConsFactors1, expand = FALSE, fill = FALSE)
	BXConsFactors$packStart(ENTConsFactorsKLA, expand = FALSE, fill = FALSE)
	BXConsFactors$packStart(LBLConsFactors2, expand = FALSE, fill = FALSE)
	BXConsFactors$packStart(ENTConsFactorsKLAError, expand = FALSE, fill = FALSE)
	BXConsFactors$packStart(LBLConsFactors3, expand = FALSE, fill = FALSE)
	BXConsFactors$packStart(ENTConsFactorsUCon, expand = FALSE, fill = FALSE)
	BXConsFactors$packStart(LBLConsFactors4, expand = FALSE, fill = FALSE)

## Axis label
LBLConsAxis <- gtkLabel("Please enter the Y-axis label for the converted oxygen consumption:")
ENTConsAxis <- gtkEntry()
ENTConsAxis$setSizeRequest(100, 20)
	BXConsAxis$packStart(LBLConsAxis, expand = FALSE, fill = FALSE)
	BXConsAxis$packStart(ENTConsAxis, expand = TRUE, fill = TRUE)

## Label
LBLConsLabel <- gtkLabel("Calculate consumed oxygen for:")
	BXConsLabel$packStart(LBLConsLabel, expand = FALSE, fill = FALSE)

## Checkboxes
CKBConsRaw <- gtkCheckButton("raw data")
	BXConsRaw$packStart(CKBConsRaw, expand = FALSE, fill = FALSE)
CKBConsRawAv <- gtkCheckButton("averaged raw data")
	BXConsRawAv$packStart(CKBConsRawAv, expand = FALSE, fill = FALSE)
CKBConsMedium <- gtkCheckButton("normalisation data")
	BXConsMedium$packStart(CKBConsMedium, expand = FALSE, fill = FALSE)
CKBConsMediumAv <- gtkCheckButton("averaged normalisation data")
	BXConsMediumAv$packStart(CKBConsMediumAv, expand = FALSE, fill = FALSE)

## Rerun
btnConsRerun <- gtkButton("Rerun calculation of consumption only")
btnConsRerun$setSizeRequest(250, 30)
btnConsRerun["visible"] <- FALSE
	BXConsRerun$packStart(btnConsRerun, expand = TRUE, fill = FALSE)

####
#### IC50 Frame
###############

BXIC50 <- gtkVBox(FALSE, 5)
BXIC50$setBorderWidth(5)
FRIC50$setBorderWidth(3)
	FRIC50$add(BXIC50)

BXIC50Body <- gtkHBox(FALSE, 10)
BXIC50Rerun <- gtkHBox(FALSE, 5)
	BXIC50$packStart(BXIC50Body, expand = TRUE, fill = TRUE)
	BXIC50$packEnd(BXIC50Rerun, expand = FALSE, fill = FALSE)
	
BXIC50Options <- gtkVBox(FALSE, 5)
SEPIC50Body <- gtkVSeparator()
BXIC50Doses <- gtkVBox(FALSE, 5)
	BXIC50Body$packStart(BXIC50Options, expand = FALSE, fill = FALSE)
	BXIC50Body$packStart(SEPIC50Body, expand = FALSE, fill = FALSE)
	BXIC50Body$packStart(BXIC50Doses, expand = TRUE, fill = TRUE)

BXIC50SelectionLabel <- gtkHBox(FALSE, 5)
BXIC50Selection <- gtkHBox(FALSE, 5)
SEPIC50Selection <- gtkLabel("")
BXIC50Timeframe <- gtkHBox(FALSE, 5)
SEPIC50Timeframe <- gtkLabel("")
BXIC50Skip <- gtkHBox(FALSE, 5)
SEPIC50Skip <- gtkLabel("")
BXIC50ModelLabel <- gtkHBox(FALSE, 5)
BXIC50Model <- gtkHBox(FALSE, 5)
SEPIC50Model <- gtkLabel("")
BXIC50AxisLabel <- gtkHBox(FALSE, 5)
BXIC50Axis <- gtkHBox(FALSE, 5)
	BXIC50Options$packStart(BXIC50SelectionLabel, expand = FALSE, fill = FALSE)
	BXIC50Options$packStart(BXIC50Selection, expand = FALSE, fill = FALSE)
	BXIC50Options$packStart(SEPIC50Selection, expand = FALSE, fill = FALSE)
	BXIC50Options$packStart(BXIC50Timeframe, expand = FALSE, fill = FALSE)
	BXIC50Options$packStart(SEPIC50Timeframe, expand = FALSE, fill = FALSE)
	BXIC50Options$packStart(BXIC50Skip, expand = FALSE, fill = FALSE)
	BXIC50Options$packStart(SEPIC50Skip, expand = FALSE, fill = FALSE)
	BXIC50Options$packStart(BXIC50ModelLabel, expand = FALSE, fill = FALSE)
	BXIC50Options$packStart(BXIC50Model, expand = FALSE, fill = FALSE)
	BXIC50Options$packStart(SEPIC50Model, expand = FALSE, fill = FALSE)
	BXIC50Options$packStart(BXIC50AxisLabel, expand = FALSE, fill = FALSE)
	BXIC50Options$packStart(BXIC50Axis, expand = FALSE, fill = FALSE)

## Selection
LBLIC50Selection <- gtkLabel("Please select the data set to be used for the IC50 calculation.")
	BXIC50SelectionLabel$packStart(LBLIC50Selection, expand = FALSE, fill = FALSE)
	
COMIC50Selection <- gtkComboBoxNewText()
	COMIC50Selection$setSizeRequest(250,25)
	BXIC50Selection$packStart(COMIC50Selection, expand = FALSE, fill = FALSE)

## Timeframe
LBLIC50Timeframe1 <- gtkLabel("Calculate IC50 from time =")
LBLIC50Timeframe2 <- gtkLabel("to time =")
LBLIC50Timeframe3 <- gtkLabel("in given input unit")
ENTIC50TimeStart <- gtkEntry()
ENTIC50TimeStart$setSizeRequest(70,20)
ENTIC50TimeEnd <- gtkEntry()
ENTIC50TimeEnd$setSizeRequest(70,20)
	BXIC50Timeframe$packStart(LBLIC50Timeframe1, expand = FALSE, fill = FALSE)
	BXIC50Timeframe$packStart(ENTIC50TimeStart, expand = FALSE, fill = FALSE)
	BXIC50Timeframe$packStart(LBLIC50Timeframe2, expand = FALSE, fill = FALSE)
	BXIC50Timeframe$packStart(ENTIC50TimeEnd, expand = FALSE, fill = FALSE)
	BXIC50Timeframe$packStart(LBLIC50Timeframe3, expand = FALSE, fill = FALSE)

## Skip
LBLIC50Skip1 <- gtkLabel("Calculate IC50 only for every")
LBLIC50Skip2 <- gtkLabel(". timepoint.")
ENTIC50Skip <- gtkEntry()
ENTIC50Skip$setSizeRequest(25,20)
	BXIC50Skip$packStart(LBLIC50Skip1, expand = FALSE, fill = FALSE)
	BXIC50Skip$packStart(ENTIC50Skip, expand = FALSE, fill = FALSE)
	BXIC50Skip$packStart(LBLIC50Skip2, expand = FALSE, fill = FALSE)

## Model Selection
LBLIC50Model <- gtkLabel("Please select which function to fit to your data.")
	BXIC50ModelLabel$packStart(LBLIC50Model, expand = FALSE, fill = FALSE)

COMIC50Model <- gtkComboBoxNewText()
	COMIC50Model$appendText("2 parameter log-logistic function")
	COMIC50Model$appendText("3 parameter log-logistic function")
	COMIC50Model$appendText("4 parameter log-logistic function")
	COMIC50Model$appendText("5 parameter log-logistic function")
	COMIC50Model$setActive(0)
	COMIC50Model$setSizeRequest(250,25)
	BXIC50Model$packStart(COMIC50Model, expand = FALSE ,fill = FALSE)

## Axis
LBLIC50Axis <- gtkLabel("Please enter a name for the X-axis on the IC50 graphics.")
	BXIC50AxisLabel$packStart(LBLIC50Axis, expand = FALSE, fill = FALSE)

ENTIC50Axis <- gtkEntry()
ENTIC50Axis$setSizeRequest(-1,20)
	BXIC50Axis$packStart(ENTIC50Axis, expand = FALSE, fill = FALSE)

## Doses
BXIC50DosesLabel1 <- gtkHBox(FALSE, 5)
BXIC50DosesLabel2 <- gtkHBox(FALSE, 5)
	BXIC50Doses$packStart(BXIC50DosesLabel1, expand = FALSE, fill = FALSE)
	BXIC50Doses$packStart(BXIC50DosesLabel2, expand = FALSE, fill = FALSE)

LBLIC50Doses1 <- gtkLabel("Please enter the concentration (without units) for each well name.")
LBLIC50Doses2 <- gtkLabel("To exclude wellnames from the IC50 calculation put -1.")
	BXIC50DosesLabel1$packStart(LBLIC50Doses1, expand = FALSE, fill = FALSE)
	BXIC50DosesLabel2$packStart(LBLIC50Doses2, expand = FALSE, fill = FALSE)

TEMPDoses <- list()

####
#### Function: Refresh lines on IC50 screen
###########################################
FRefreshIC50Doses <- function(T) {
	
	oldDoses <- BXIC50Doses$getChildren()
	if (length(oldDoses) >= 3) {
		FCheckIC50Doses(Template)
		for (i in 3:length(oldDoses)) {
			gtkWidgetDestroy(oldDoses[[i]])
		}
	}
	
	doses <- c()
	names <- levels(as.factor(T$Name))
	
	TEMPDoses <<- list()
	for (i in 1:length(names)) {
		TEMPDoses[[names[i]]] <<- list()
		TEMPDoses[[names[i]]]$box <<- gtkHBox(FALSE, 5)
		TEMPDoses[[names[i]]]$label <<- gtkLabel(paste(names[i], ":", sep=""))
		gtkWidgetSetSizeRequest(TEMPDoses[[names[i]]]$label, 100, 20)		
		TEMPDoses[[names[i]]]$entry <<- gtkEntry()
		gtkEntrySetText(TEMPDoses[[names[i]]]$entry, tryCatch(as.numeric(names[i]), warning = function(W) return(-1)))
		if (length(which(SETAnalysisOpt$IC50doses$Name == names[i])) >= 1) gtkEntrySetText(TEMPDoses[[names[i]]]$entry, SETAnalysisOpt$IC50doses$Dose[which(SETAnalysisOpt$IC50doses$Name == names[i])])
		gtkWidgetSetSizeRequest(TEMPDoses[[names[i]]]$entry, 50, 20)
			gtkBoxPackStart(TEMPDoses[[names[i]]]$box, TEMPDoses[[names[i]]]$label, expand = FALSE, fill = FALSE)
			gtkBoxPackStart(TEMPDoses[[names[i]]]$box, TEMPDoses[[names[i]]]$entry, expand = FALSE, fill = FALSE)
			BXIC50Doses$packStart(TEMPDoses[[names[i]]]$box, expand = FALSE, fill = FALSE)
		dose <- as.numeric(gtkEntryGetText(TEMPDoses[[names[i]]]$entry))
		doses <- c(doses, dose)
	}
	SETAnalysisOpt$IC50doses <<- data.frame(Name = names, Dose = doses)
}
## Rerun
btnIC50Rerun <- gtkButton("Rerun IC50 calculation only.")
btnIC50Rerun$setSizeRequest(250, 30)
btnIC50Rerun["visible"] <- FALSE
	BXIC50Rerun$packStart(btnIC50Rerun, expand = TRUE, fill = FALSE)


#########################################################################################
#### Analysis Options interactive
#########################################################################################

####
#### Signal: Calculate IC50
###########################
gSignalConnect(CKBSelectionIC50, "toggled", quote({
	if (CKBSelectionIC50$getActive() == TRUE) {
		## Settings
		SETAnalysisOpt$IC50calc <- TRUE
		## Layouts
		FRIC50["visible"] <- TRUE
		BXPlotGeneralSubIC50Time["visible"] <- TRUE
		BXPlotGeneralSubIC50Fits["visible"] <- TRUE
		FRMiniaturIC50["visible"] <- TRUE
		FROutIC50["visible"] <- TRUE
	} else {
		## Settings
		SETAnalysisOpt$IC50calc <- FALSE
		## Layouts
		FRIC50["visible"] <- FALSE
		BXPlotGeneralSubIC50Time["visible"] <- FALSE
		BXPlotGeneralSubIC50Fits["visible"] <- FALSE
		FRMiniaturIC50["visible"] <- FALSE
		FROutIC50["visible"] <- FALSE
		## Restrictions:
		ListOutputLayouts[["Data_IC50"]][["ckb"]]$setActive(is.active = FALSE)
	}
}))

####
#### Signal: Selection raw average
##################################
gSignalConnect(CKBSelectionRawAv, "toggled", quote({
	if (CKBSelectionRawAv$getActive() == TRUE) {
		## Settings
		SETAnalysisOpt$raw[2] <- TRUE
		TEMPIC50Selection["Data_rawAv"] <- TRUE
		## Layouts
		BXSmoothRawAv["visible"] <- TRUE
		BXSlopeRawAv["visible"] <- TRUE
		BXConsRawAv["visible"] <- as.logical(SETAnalysisOpt$slopeSelection["Data_rawAv"])
		BXPlotGeneralSubRawAv["visible"] <- TRUE
		FROutRawAv["visible"] <- TRUE
		FROutRawSd["visible"] <- as.logical(SETAnalysisOpt$SD)
		btnRawAvShow["visible"] <- TRUE
	} else {
		## Settings
		SETAnalysisOpt$raw[2] <- FALSE
		TEMPIC50Selection["Data_rawAv"] <- FALSE
		## Layouts
		BXSmoothRawAv["visible"] <- FALSE
		BXSlopeRawAv["visible"] <- FALSE
		BXConsRawAv["visible"] <- FALSE
		BXPlotGeneralSubRawAv["visible"] <- FALSE
		FROutRawAv["visible"] <- FALSE
		FROutRawSd["visible"] <- FALSE
		btnRawAvShow["visible"] <- FALSE
		## Restrictions
		CKBSmRawAv$setActive(is.active = FALSE)
		CKBSlopeRawAv$setActive(is.active = FALSE)
		CKBConsRawAv$setActive(is.active = FALSE)
		ListOutputLayouts[["Data_rawAv"]][["ckb"]]$setActive(is.active = FALSE)
		ListOutputLayouts[["Data_rawSd"]][["ckb"]]$setActive(is.active = FALSE)
	}
	FRefreshIC50Selection()
	#print(SETAnalysisOpt$raw)
}))

####
#### Signal: Selection Medium cal
#################################
gSignalConnect(CKBSelectionMedium, "toggled", quote({
	if (CKBSelectionMedium$getActive() == TRUE) {
		## Settings
		SETAnalysisOpt$medium[1] <- TRUE
		TEMPIC50Selection["Data_medium"] <- TRUE
		TEMPFormatSelectData["Data_medium"] <- TRUE
		## Lyouts
		FRMediumCal["visible"] <- TRUE
		BXSmoothMedium["visible"] <- TRUE
		BXSlopeMedium["visible"] <- TRUE
		BXConsMedium["visible"] <- as.logical(SETAnalysisOpt$slopeSelection["Data_normalisation"])
		BXPlotGeneralSubMedium["visible"] <- TRUE
		FROutMedium["visible"] <- TRUE
		FRMiniaturMedium["visible"] <- TRUE
		## Restrictions - none
	} else {
		## Settings
		SETAnalysisOpt$medium[1] <- FALSE
		TEMPIC50Selection["Data_medium"] <- FALSE
		TEMPFormatSelectData["Data_medium"] <- FALSE
		## Layouts
		FRMediumCal["visible"] <- FALSE
		BXSmoothMedium["visible"] <- FALSE
		BXSlopeMedium["visible"] <- FALSE
		BXConsMedium["visible"] <- FALSE
		BXPlotGeneralSubMedium["visible"] <- FALSE
		FROutMedium["visible"] <- FALSE
		FRMiniaturMedium["visible"] <- FALSE
		## Restrictions
		CKBSelectionMediumAv$setActive(is.active = FALSE)
		CKBSmMedium$setActive(is.active = FALSE)
		CKBSlopeMedium$setActive(is.active = FALSE)
		CKBConsMedium$setActive(is.active = FALSE)
		ListOutputLayouts[["Data_medium"]][["ckb"]]$setActive(is.active = FALSE)
	}
	FRefreshIC50Selection()
	FRefreshFormattingSelection()
	#print(SETAnalysisOpt$medium)
}))

####
#### Signal: Selection Medium cal average
#########################################
gSignalConnect(CKBSelectionMediumAv, "toggled", quote({
	if (CKBSelectionMediumAv$getActive() == TRUE) {
		## Settings
		SETAnalysisOpt$medium[2] <- TRUE
		TEMPIC50Selection["Data_mediumAv"] <- TRUE
		## Layouts
		BXSmoothMediumAv["visible"] <- TRUE
		BXSlopeMediumAv["visible"] <- TRUE
		BXConsMediumAv["visible"] <- as.logical(SETAnalysisOpt$slopeSelection["Data_mediumAv"])
		BXPlotGeneralSubMediumAv["visible"] <- TRUE
		FROutMediumAv["visible"] <- TRUE
		FROutMediumSd["visible"] <- as.logical(SETAnalysisOpt$SD)
		btnMediumAvShow["visible"] <- TRUE
		## Restrictions
		CKBSelectionMedium$setActive(is.active = TRUE)
	} else {
		## Settings
		SETAnalysisOpt$medium[2] <- FALSE
		TEMPIC50Selection["Data_mediumAv"] <- FALSE
		## Layouts
		BXSmoothMediumAv["visible"] <- FALSE
		BXSlopeMediumAv["visible"] <- FALSE
		BXConsMediumAv["visible"] <- FALSE
		BXPlotGeneralSubMediumAv["visible"] <- FALSE
		FROutMediumAv["visible"] <- FALSE
		FROutMediumSd["visible"] <- FALSE
		btnMediumAvShow["visible"] <- FALSE
		## Restrictions
		CKBSmMediumAv$setActive(is.active = FALSE)
		CKBSlopeMediumAv$setActive(is.active = FALSE)
		CKBConsMediumAv$setActive(is.active = FALSE)
		ListOutputLayouts[["Data_mediumAv"]][["ckb"]]$setActive(is.active = FALSE)
		ListOutputLayouts[["Data_mediumSd"]][["ckb"]]$setActive(is.active = FALSE)
	}
	FRefreshIC50Selection()
	#print(SETAnalysisOpt$medium)
}))

####
#### Signal: Print timepoints
#############################
gSignalConnect(btnPrintTimepoints, "clicked", quote({
	if (Data_input[1,1] != "empty") {
		Temp <- data.frame("input unit" = Data_input$X, "output unit" = Data_input$X * SETDataLayout$timescale / SETGraphOpt$XScale)
		print(Temp)
		rm(Temp)
	} else {
		print("Please import data first!")
	}
}))

####
#### Signal: Print dimensions
#############################
gSignalConnect(btnPrintDimensions, "clicked", quote({
	if (Data_input[1,1] != "empty") {
		print(paste("Data_input", ":", dim(Data_input)[1], "rows", dim(Data_input)[2], "columns", sep=" "))
		for (i in 1:length(ListDataSets)) {
			if (length(get(names(ListDataSets)[i]))[1] != 0) {
				print(paste(names(ListDataSets)[i], ":", dim(get(names(ListDataSets)[i]))[1], "rows", dim(get(names(ListDataSets)[i]))[2], "columns", sep=" "))
			}
		}
	} else {
		print("Please import data first!")
	}
}))

####
#### Signal: Medium name
########################
ERRMediumName <- gtkLabel("-- Given name for the normalisation is not in the labels list! --")
	ERRMediumName["visible"] <- FALSE
	BXErrormessages$packStart(ERRMediumName, expand = TRUE, fill = TRUE)

gSignalConnect(ENTMediumCalName, "activate", quote({
	text <- ENTMediumCalName$getText()
	if (Template[1,1] != "empty" & SETAnalysisOpt$medium[1] == TRUE) {
		for (i in 1:length(Template$X)) {
			if (Template$Name[i] == text) {
				SETAnalysisOpt$name <- text
				ERRMediumName["visible"] <- FALSE
				break
			}
			ERRMediumName["visible"] <- TRUE
		}
	} else { #no template loaded yet
		SETAnalysisOpt$name <- text
		ERRMediumName["visible"] <- FALSE
	}
}))

####
#### Signal: Medium calibration target
######################################
ERRMediumTarget <- gtkLabel("-- Please enter a number for the normalisation target! --")
	ERRMediumTarget["visible"] <- FALSE
	BXErrormessages$packStart(ERRMediumTarget, expand = TRUE, fill = TRUE)
	
gSignalConnect(ENTMediumCalTarget, "activate", quote({
	if (SETAnalysisOpt$medium[1] == TRUE) {
		text <- tryCatch(as.numeric(ENTMediumCalTarget$getText()), warning = function(W) ERRMediumTarget["visible"] <- TRUE)
		if (is.numeric(text) == TRUE) {
			SETAnalysisOpt$medTarget <- text
			ERRMediumTarget["visible"] <- FALSE
		}
	} else {
		ERRMediumTarget["visible"] <- FALSE
	}
}))

####
#### Signal: measurement unit for time?
#######################################
####
#### Function: Choice of input timescale
#########################################
FInputTimescale <- function() {
	if (btnDataLayoutTimesec$getActive() == TRUE) {
		SETDataLayout$timescale <<- 1
	} else if (btnDataLayoutTimemin$getActive() == TRUE) {
		SETDataLayout$timescale <<- 60
	} else if (btnDataLayoutTimeh$getActive() == TRUE) {
		SETDataLayout$timescale <<- 3600
	} else if (btnDataLayoutTimeday$getActive() == TRUE) {
		SETDataLayout$timescale <<- 86400
	}
	#print(SETDataLayout$timescale)
}

gSignalConnect(btnDataLayoutTimesec, "toggled", quote(FInputTimescale()))
gSignalConnect(btnDataLayoutTimemin, "toggled", quote(FInputTimescale()))
gSignalConnect(btnDataLayoutTimeh, "toggled", quote(FInputTimescale()))

####
#### Signal: Scale for X-axis
#############################
####
#### Function: Choice of output timescale
#########################################
FOutputTimescale <- function() {
	if (btnPlotAxesH1sec$getActive() == TRUE) {
		SETGraphOpt$XScale <<- 1
		ENTPlotAxesH2$setText("Time [sec]")
	} else if (btnPlotAxesH1min$getActive() == TRUE) {
		SETGraphOpt$XScale <<- 60
		ENTPlotAxesH2$setText("Time [min]")
	} else if (btnPlotAxesH1h$getActive() == TRUE) {
		SETGraphOpt$XScale <<- 3600
		ENTPlotAxesH2$setText("Time [h]")
	} else if (btnPlotAxesH1day$getActive() == TRUE) {
		SETGraphOpt$XScale <<- 86400
		ENTPlotAxesH2$setText("Time [day]")
	}
	#print(SETGraphOpt$XScale)
}

gSignalConnect(btnPlotAxesH1sec, "toggled", quote(FOutputTimescale()))
gSignalConnect(btnPlotAxesH1min, "toggled", quote(FOutputTimescale()))
gSignalConnect(btnPlotAxesH1h, "toggled", quote(FOutputTimescale()))

####
#### Signal: Selection for formatting changed
#############################################
SelectionHandlerFormatting <- gSignalConnect(COMFormatSelectData, "changed", after = TRUE, quote({
	if (COMFormatSelectData$getActive() != -1) {
		SETAnalysisOpt$Formatting <- names(ListDataSets[which(ListDataSets == COMFormatSelectData$getActiveText())])
	}
	for (i in 1:length(FormattingResultsInterfaces)) FormattingResultsInterfaces[[i]]["visible"] <- FALSE
	for (i in 1:length(TEMPFormatSelectData)) if (class(ListOutputLayouts[[names(TEMPFormatSelectData)[i]]][["ckb"]])[1] == "GtkCheckButton") ListOutputLayouts[[names(TEMPFormatSelectData)[i]]][["ckb"]]$setActive(is.active = FALSE)
	if (length(SETAnalysisOpt$Formatting) == 0) {
	} else if (SETAnalysisOpt$Formatting == "Data_raw") {
		BXPlotGeneralSubRawOrig["visible"] <- TRUE
		FROutRawOrig["visible"] <- TRUE
		btnRawOrigShow["visible"] <- TRUE
	} else if (SETAnalysisOpt$Formatting == "Data_oxoDish") {
		BXPlotGeneralSubOxoDishOrig["visible"] <- TRUE
		FROutOxoDishOrig["visible"] <- TRUE
		btnOxoDishOrigShow["visible"] <- TRUE
	} else if (SETAnalysisOpt$Formatting == "Data_oxoPlate") {
		BXPlotGeneralSubOxoPlateOrig["visible"] <- TRUE
		FROutOxoPlateOrig["visible"] <- TRUE
		btnOxoPlateOrigShow["visible"] <- TRUE
	} else if (SETAnalysisOpt$Formatting == "Data_hydroPlate") {
		BXPlotGeneralSubHydroPlateOrig["visible"] <- TRUE
		FROutHydroPlateOrig["visible"] <- TRUE
		btnHydroPlateOrigShow["visible"] <- TRUE
	} else if (SETAnalysisOpt$Formatting == "Data_medium") {
		BXPlotGeneralSubMediumOrig["visible"] <- TRUE
		FROutMediumOrig["visible"] <- TRUE
		btnMediumOrigShow["visible"] <- TRUE
	}
	FCheckAnalysisOpt(c(ENTFormatCutXStart, ENTFormatCutXEnd, ENTFormatTransformXm, ENTFormatTransformXb, ENTFormatTransformY))
	#print(SETAnalysisOpt$Formatting)
}))

####
#### Signals: ParametersFor Formatting
######################################
ERRFormatNumeric <- gtkLabel("-- Please only enter numerical values for the formatting parameters! --")
	ERRFormatNumeric["visible"] <- FALSE
	BXErrormessages$packStart(ERRFormatNumeric, expand = FALSE, fill = FALSE)
ERRFormatSensorConflict <- gtkLabel("-- You are about to remove the stop timepoint of the sensor correction! --")
	ERRFormatSensorConflict["visible"] <- FALSE
	BXErrormessages$packStart(ERRFormatSensorConflict, expand = FALSE, fill = FALSE)

checkFormatNumeric <- c("t1" = TRUE, "t2" = TRUE, "m" = TRUE, "b" = TRUE, "y" = TRUE)
FFormattingGetParam <- function(name, ent) {
	text <- tryCatch(as.numeric(ent$getText()), warning = function(W) { checkFormatNumeric[name] <<- FALSE })
	if (is.numeric(text) == TRUE & is.na(text) == FALSE) {
		SETAnalysisOpt$FormattingParams[name] <<- text
		checkFormatNumeric[name] <<- TRUE
	} else {
		checkFormatNumeric[name] <<- FALSE
	}
	if (length(SETAnalysisOpt$Formatting) != 0) {
			for (i in 1:length(checkFormatNumeric)) {
				if (checkFormatNumeric[i] == FALSE) {
					ERRFormatNumeric["visible"] <- TRUE
					break
				} else ERRFormatNumeric["visible"] <- FALSE
			}
	} else {
		ERRFormatNumeric["visible"] <- FALSE
	}
}
gSignalConnect(ENTFormatCutXStart, "activate", quote({
	FFormattingGetParam("t1", ENTFormatCutXStart)
	if (length(SETAnalysisOpt$Formatting) != 0 & SETAnalysisOpt$oxoDish[1] == TRUE) {
		if (checkFormatNumeric["t2"] == TRUE & checkFormatNumeric["t1"] == TRUE & SETAnalysisOpt$Formatting == "Data_raw") {
			timepoint <- SETAnalysisOpt$range["last"] * SETDataLayout$timescale / SETGraphOpt$XScale
			if (timepoint < SETAnalysisOpt$FormattingParam["t1"] | timepoint > SETAnalysisOpt$FormattingParam["t2"]) ERRFormatSensorConflict["visible"] <- TRUE
			else ERRFormatSensorConflict["visible"] <- FALSE
		} else ERRFormatSensorConflict["visible"] <- FALSE
	} else ERRFormatSensorConflict["visible"] <- FALSE
}))
gSignalConnect(ENTFormatCutXEnd, "activate", quote({
	FFormattingGetParam("t2", ENTFormatCutXEnd)
	if (length(SETAnalysisOpt$Formatting) != 0 & SETAnalysisOpt$oxoDish[1] == TRUE) {
		if (checkFormatNumeric["t2"] == TRUE & checkFormatNumeric["t1"] == TRUE & SETAnalysisOpt$Formatting == "Data_raw") {
			timepoint <- SETAnalysisOpt$range["last"] * SETDataLayout$timescale / SETGraphOpt$XScale
			if (timepoint < SETAnalysisOpt$FormattingParam["t1"] | timepoint > SETAnalysisOpt$FormattingParam["t2"]) 	ERRFormatSensorConflict["visible"] <- TRUE
			else ERRFormatSensorConflict["visible"] <- FALSE
		} else ERRFormatSensorConflict["visible"] <- FALSE
	} else ERRFormatSensorConflict["visible"] <- FALSE
}))
gSignalConnect(ENTFormatTransformXm, "activate", quote(FFormattingGetParam("m", ENTFormatTransformXm)))
gSignalConnect(ENTFormatTransformXb, "activate", quote(FFormattingGetParam("b", ENTFormatTransformXb)))
gSignalConnect(ENTFormatTransformY, "activate", quote(FFormattingGetParam("y", ENTFormatTransformY)))

####
#### Signal: Average Mode
#########################
####
#### Function: Selection of Average Mode
########################################
FAverageMode <- function() {
	if (btnAverageName$getActive() == TRUE) {
		SETAnalysisOpt$average <<- "Name"
	} else if (btnAverageNumber$getActive() == TRUE) {
		SETAnalysisOpt$average <<- "Number"
	} else if (btnAverageColour$getActive() == TRUE) {
		SETAnalysisOpt$average <<- "Colour"
	}
	#print(SETAnalysisOpt$average)
}

gSignalConnect(btnAverageName, "toggled", quote(FAverageMode()))
gSignalConnect(btnAverageNumber, "toggled", quote(FAverageMode()))

####
#### Signals: Checkboxes for Smoothing
######################################
####
#### Function: Smoothing check
##############################
FSignalCheckSmooth <- function(CKB, SET, Sub, btn, FROut) {
	value <- CKB$getActive()
	SETAnalysisOpt$smoothSelection[SET] <<- value
	Sub["visible"] <- value
	btn["visible"] <- value
	FROut["visible"] <- value
	if (value == FALSE)	ListOutputLayouts[[paste0(SET, "Sm")]][[3]]$setActive(is.active = FALSE)
	TEMPIC50Selection[paste0(SET, "Sm")] <<- value
	FRefreshIC50Selection()
	#print(SETAnalysisOpt$smoothSelection)
	#print(SETDataOutput$export)
}
gSignalConnect(CKBSmRaw, "toggled", after = TRUE, quote(FSignalCheckSmooth(CKB = CKBSmRaw, SET = "Data_raw", Sub = BXPlotGeneralSubRawSm, btn = btnRawSmShow, FROut = FROutRawSm)))
gSignalConnect(CKBSmRawAv, "toggled", after = TRUE, quote(FSignalCheckSmooth(CKB = CKBSmRawAv, SET = "Data_rawAv", Sub = BXPlotGeneralSubRawAvSm, btn = btnRawAvSmShow, FROut = FROutRawAvSm)))
gSignalConnect(CKBSmMedium, "toggled", after = TRUE, quote(FSignalCheckSmooth(CKB = CKBSmMedium, SET = "Data_medium", Sub = BXPlotGeneralSubMediumSm, btn = btnMediumSmShow, FROut = FROutMediumSm)))
gSignalConnect(CKBSmMediumAv, "toggled", after = TRUE, quote(FSignalCheckSmooth(CKB = CKBSmMediumAv, SET = "Data_mediumAv", Sub = BXPlotGeneralSubMediumAvSm, btn = btnMediumAvSmShow, FROut = FROutMediumAvSm)))

####
#### Signals: Checkboxes for Slope
##################################
####
#### Function: Slope check
##########################
FSignalCheckSlope <- function(CKB, SET, Sub, btn, FROut, FROutSd, BXCons, CKBCons) {
	value <- CKB$getActive()
	SETAnalysisOpt$slopeSelection[SET] <<- value
	Sub["visible"] <- value
	btn["visible"] <- value
	FROut["visible"] <- value
	FROutSd["visible"] <- value
	if (length(class(BXCons)) != 1) BXCons["visible"] <- value
	if (value == FALSE)	{
		ListOutputLayouts[[paste0(SET, "Slope")]][["ckb"]]$setActive(is.active = FALSE)
		ListOutputLayouts[[paste0(SET, "SlopeSd")]][["ckb"]]$setActive(is.active = FALSE)
		if (length(class(CKBCons)) != 1) CKBCons$setActive(is.active = FALSE)
	}
	#print(SETAnalysisOpt$slopeSelection)
	#print(SETDataOutput$export)
}
gSignalConnect(CKBSlopeRaw, "toggled", after = TRUE, quote(FSignalCheckSlope(CKB = CKBSlopeRaw, SET = "Data_raw", Sub = BXPlotGeneralSubRawSlope, btn = btnRawSlopeShow, FROut = FROutRawSlope, FROutSd = FROutRawSlopeSd, BXCons = BXConsRaw, CKBCons = CKBConsRaw)))
gSignalConnect(CKBSlopeRawAv, "toggled", after = TRUE, quote(FSignalCheckSlope(CKB = CKBSlopeRawAv, SET = "Data_rawAv", Sub = BXPlotGeneralSubRawAvSlope, btn = btnRawAvSlopeShow, FROut = FROutRawAvSlope, FROutSd = FROutRawAvSlopeSd, BXCons = BXConsRawAv, CKBCons = CKBConsRawAv)))
gSignalConnect(CKBSlopeMedium, "toggled", after = TRUE, quote(FSignalCheckSlope(CKB = CKBSlopeMedium, SET = "Data_medium", Sub = BXPlotGeneralSubMediumSlope, btn = btnMediumSlopeShow, FROut = FROutMediumSlope, FROutSd = FROutMediumSlopeSd, BXCons = BXConsMedium, CKBCons = CKBConsMedium)))
gSignalConnect(CKBSlopeMediumAv, "toggled", after = TRUE, quote(FSignalCheckSlope(CKB = CKBSlopeMediumAv, SET = "Data_mediumAv", Sub = BXPlotGeneralSubMediumAvSlope, btn = btnMediumAvSlopeShow, FROut = FROutMediumAvSlope, FROutSd = FROutMediumAvSlopeSd, BXCons = BXConsMediumAv, CKBCons = CKBConsMediumAv)))

####
#### Signal: Checkboxes for Consumption
#######################################
####
#### Function: Consumption Check
################################
FSignalCheckCons <- function(CKB, SET, Sub, btn, FROut, FROutSd) {
	value <- CKB$getActive()
	SETAnalysisOpt$ConsSelection[SET] <<- value
	Sub["visible"] <- value
	btn["visible"] <- value
	FROut["visible"] <- value
	FROutSd["visible"] <- value
	TEMPIC50Selection[paste0(SET, "Cons")] <<- value
	FRefreshIC50Selection()
	if (value == FALSE) {
		ListOutputLayouts[[paste0(SET, "Cons")]][["ckb"]]$setActive(is.active = FALSE)
		ListOutputLayouts[[paste0(SET, "ConsSd")]][["ckb"]]$setActive(is.active = FALSE)
	}
}
gSignalConnect(CKBConsRaw, "toggled", after = TRUE, quote(FSignalCheckCons(CKB = CKBConsRaw, SET = "Data_raw", Sub = BXPlotGeneralSubRawCons, btn = btnRawConsShow, FROut = FROutRawCons, FROutSd = FROutRawConsSd)))
gSignalConnect(CKBConsRawAv, "toggled", after = TRUE, quote(FSignalCheckCons(CKB = CKBConsRawAv, SET = "Data_rawAv", Sub = BXPlotGeneralSubRawAvCons, btn = btnRawAvConsShow, FROut = FROutRawAvCons, FROutSd = FROutRawAvConsSd)))
gSignalConnect(CKBConsMedium, "toggled", after = TRUE, quote(FSignalCheckCons(CKB = CKBConsMedium, SET = "Data_medium", Sub = BXPlotGeneralSubMediumCons, btn = btnMediumConsShow,  FROut = FROutMediumCons, FROutSd = FROutMediumConsSd)))
gSignalConnect(CKBConsMediumAv, "toggled", after = TRUE, quote(FSignalCheckCons(CKB = CKBConsMediumAv, SET = "Data_mediumAv", Sub = BXPlotGeneralSubMediumAvCons, btn = btnMediumAvConsShow, FROut = FROutMediumAvCons, FROutSd = FROutMediumAvConsSd)))

####
#### Signal: Smoothing
######################
ERRSmooth <- gtkLabel("-- Please enter an odd number for the curve smoothing! --")
	ERRSmooth["visible"] <- FALSE
	BXErrormessages$packStart(ERRSmooth, expand = TRUE, fill = TRUE)
ERRSmoothSize <- gtkLabel("-- Your imported data doesn't contain enough time points for smoothing! --")
	ERRSmoothSize["visible"] <- FALSE
	BXErrormessages$packStart(ERRSmoothSize, expand = TRUE, fill = TRUE)
errsmooth <- c(FALSE, FALSE)

gSignalConnect(ENTSmoothOption, "activate", quote({
	if (SETAnalysisOpt$smoothParam["calc"] == TRUE) {
		text <- tryCatch(as.numeric(ENTSmoothOption$getText()), warning = function(W) {
			ERRSmooth["visible"] <- TRUE
			errsmooth[1] <- TRUE
			ERRSmoothSize["visible"] <- FALSE
		})
		if (is.numeric(text) == TRUE & is.na(text) == FALSE & text %% 2 == 1) {
			errsmooth[1] <- FALSE
			if (errsmooth[1] == FALSE & errsmooth[2] == FALSE)	ERRSmooth["visible"] <- FALSE
			SETAnalysisOpt$smoothParam[["range"]] <- text
			if (Data_input[1,1] != "empty" & dim(Data_input)[1] <= text-1) ERRSmoothSize["visible"] <- TRUE
			else ERRSmoothSize["visible"] <- FALSE
		} else {
			ERRSmooth["visible"] <- TRUE
			errsmooth[1] <- TRUE
			ERRSmoothSize["visible"] <- FALSE
		}
	} else {
		errsmooth[1] <- FALSE
		if (errsmooth[1] == FALSE & errsmooth[2] == FALSE)	ERRSmooth["visible"] <- FALSE
		ERRSmoothSize["visible"] <- FALSE
	}
	#print(SETAnalysisOpt$slopeParam)
}))

####
#### Signal: Slope Smoothing
############################
####
#### Function: Calculate Frames
###############################
ERRSlopeSize <- gtkLabel("-- Your imported data doesn't contain enough time points for the slope calculation! --")
	ERRSlopeSize["visible"] <- FALSE
	BXErrormessages$packStart(ERRSlopeSize, expand = TRUE, fill = TRUE)

FCalculateSlopeFrames <- function() {
	## determine border
	SlopeEdge <<- ((SETAnalysisOpt$slopeParam["average"] - 1) / 2) ## average frame to one side
	SlopeFrame <<- (SETAnalysisOpt$slopeParam["points"] * 2 + 1) + (2 * SlopeEdge)
	SlopeBorder <<- SETAnalysisOpt$slopeParam["points"] + SlopeEdge ## fitting frame to one side
	if (Data_input[1,1] != "empty" & dim(Data_input)[1] <= SlopeFrame-1) ERRSlopeSize["visible"] <- TRUE
	else ERRSlopeSize["visible"] <- FALSE
}

gSignalConnect(ENTSlopeSmooth, "activate", quote({
	if (SETAnalysisOpt$slopeParam[["calc"]] == TRUE) {
		text <- tryCatch(as.numeric(ENTSlopeSmooth$getText()), warning = function(W) {
			ERRSmooth["visible"] <- TRUE
			errsmooth[2] <- TRUE
		})
		if (is.numeric(text) == TRUE & is.na(text) == FALSE & text %% 2 != 1) {
			ERRSmooth["visible"] <- TRUE
			errsmooth[2] <- TRUE
		} else if (is.numeric(text) == TRUE & is.na(text) == FALSE) {
			errsmooth[2] <- FALSE
			if (errsmooth[1] == FALSE & errsmooth[2] == FALSE)	ERRSmooth["visible"] <- FALSE
			SETAnalysisOpt$slopeParam[["average"]] <- text
			FCalculateSlopeFrames()
		} else {
			ERRSmooth["visible"] <- TRUE
			errsmooth[2] <- TRUE
		}
	} else {
		if (errsmooth[1] == FALSE) ERRSmooth["visible"] <- FALSE
		ERRSlopeSize["visible"] <- FALSE
	}
	#print(SETAnalysisOpt$slopeParam)
}))

####
#### Signal: Slope Pointnumber
##############################
ERRSlopePoints <- gtkLabel("-- Please enter a number for the slope fitting points! --")
	ERRSlopePoints["visible"] <- FALSE
	BXErrormessages$packStart(ERRSlopePoints, expand = TRUE, fill = TRUE)

gSignalConnect(ENTSlopePoints, "activate", quote({
	if (SETAnalysisOpt$slopeParam[["calc"]] == TRUE) {
		text <- tryCatch(as.numeric(ENTSlopePoints$getText()), warning = function(W) {
			ERRSlopePoints["visible"] <- TRUE
			ERRSlopeSize["visible"] <- FALSE
		})
		if (is.numeric(text) == TRUE & is.na(text) == FALSE) {
			ERRSlopePoints["visible"] <- FALSE
			SETAnalysisOpt$slopeParam[["points"]] <- text
			FCalculateSlopeFrames()
		} else {
			ERRSlopePoints["visible"] <- TRUE
			ERRSlopeSize["visible"] <- FALSE
		}
	} else {
		ERRSlopePoints["visible"] <- FALSE
		ERRSlopeSize["visible"] <- FALSE
	}
	#print(SETAnalysisOpt$slopeParam)
}))

####
#### Signal: Import Calibration results
#######################################
gSignalConnect(CKBConsLoadCal, "toggled", quote({
	if (CKBConsLoadCal$getActive() == TRUE) {
		ENTConsFactorsKLA["editable"] <- FALSE
		ENTConsLoadCal["visible"] <- TRUE
		btnConsLoadCal["visible"] <- TRUE
		SETAnalysisOpt$ConsParam["load"] <- TRUE
	} else {
		ENTConsFactorsKLA["editable"] <- TRUE
		ENTConsLoadCal["visible"] <- FALSE
		btnConsLoadCal["visible"] <- FALSE
		SETAnalysisOpt$ConsParam["load"] <- FALSE
		children <- BXConsSelectCal$getChildren()
		if (length(children) > 1) {
			for (i in 2:length(children)) children[[i]]$destroy()
		}
	}
}))

####
#### Signal: Filename for calibration entered
#############################################
ERRLoadingCalibration <- gtkLabel("Loading calibration file ...")
ERRLoadingCalibration["visible"] <- FALSE
	BXErrormessages$packStart(ERRLoadingCalibration, expand = FALSE, fill = FALSE)

gSignalConnect(ENTConsLoadCal, "activate", quote({
	path <- ENTConsLoadCal$getText()
	if (file.exists(path) == TRUE) {
		con <- file(path, open = "r")
		SETAnalysisOpt$ConsCharacters["file"] <- path
		ERRLoadingCalibration["visible"] <- TRUE
		CalibrationResults <- unserialize(con)
		close(con)
		FProduceKLAValueChoice()
		ERRLoadingCalibration["visible"] <- FALSE
	}
}))

####
#### Signal: Choose file for Calibration to Load
################################################
gSignalConnect(btnConsLoadCal, "clicked", quote({
	FFileChooserOpen(ENTConsLoadCal)
	gSignalEmit(ENTConsLoadCal, "activate")
}))

####
#### Signal: Saturated oxygen value
###################################
ERRConsSat <- gtkLabel("-- Please enter a number for the saturated oxygen concentration! --")
	ERRConsSat["visible"] <- FALSE
	BXErrormessages$packStart(ERRConsSat, expand = FALSE, fill = FALSE)

gSignalConnect(ENTConsSat, "activate", quote({
	if (SETAnalysisOpt$ConsParam["calc"] == TRUE) {
		text <- tryCatch(as.numeric(ENTConsSat$getText()), warning = function(W) ERRConsSat["visible"] <- TRUE)
		if (is.numeric(text) == TRUE & is.na(text) == FALSE) {
			ERRConsSat["visible"] <- FALSE
			SETAnalysisOpt$ConsParam["saturation"] <- text
		} else ERRConsSat["visible"] <- TRUE
	} else {
		ERRConsSat["visible"] <- FALSE
	}
}))
gSignalConnect(btnConsSatMedium, "clicked", quote({
	if (SETAnalysisOpt$medium[1] == TRUE & SETAnalysisOpt$ConsParam[1] == TRUE) {
		ENTConsSat$setText(SETAnalysisOpt$medTarget)
	}
}))

####
#### Signal: kLa
################
ERRConsKLA <- gtkLabel("-- Please enter a number for the kLa value! --")
	ERRConsKLA["visible"] <- FALSE
	BXErrormessages$packStart(ERRConsKLA, expand = FALSE, fill = FALSE)

gSignalConnect(ENTConsFactorsKLA, "activate", quote({
	if (SETAnalysisOpt$ConsParam["calc"] == TRUE) {
		text <- tryCatch(as.numeric(ENTConsFactorsKLA$getText()), warning = function(W) ERRConsKLA["visible"] <- TRUE)
		if (is.numeric(text) == TRUE & is.na(text) == FALSE) {
			ERRConsKLA["visible"] <- FALSE
			SETAnalysisOpt$ConsParam["kLa"] <- text
		} else ERRConsKLA["visible"] <- TRUE
	} else {
		ERRConsKLA["visible"] <- FALSE
	}
}))

####
#### Signal: kLa error
######################
ERRConsKLAError <- gtkLabel("-- Please enter a number for the error of the kLa value! --")
	ERRConsKLAError["visible"] <- FALSE
	BXErrormessages$packStart(ERRConsKLAError, expand = FALSE, fill = FALSE)

gSignalConnect(ENTConsFactorsKLAError, "activate", quote({
	if (SETAnalysisOpt$ConsParam["calc"] == TRUE) {
		text <- tryCatch(as.numeric(ENTConsFactorsKLAError$getText()), warning = function(W) ERRConsKLAError["visible"] <- TRUE)
		if (is.numeric(text) == TRUE & is.na(text) == FALSE) {
			ERRConsKLAError["visible"] <- FALSE
			SETAnalysisOpt$ConsParam["kLaerror"] <- text
		} else ERRConsKLAError["visible"] <- TRUE
	} else {
		ERRConsKLAError["visible"] <- FALSE
	}
}))

####
#### Signal: Consmuption Conversion factor
##########################################
ERRConsUCon <- gtkLabel("-- Please enter a number for the unit conversion factor used for the oxygen consumption! --")
	ERRConsUCon["visible"] <- FALSE
	BXErrormessages$packStart(ERRConsUCon, expand = FALSE, fill = FALSE)

gSignalConnect(ENTConsFactorsUCon, "activate", quote({
	if (SETAnalysisOpt$ConsParam["calc"] == TRUE) {
		text <- tryCatch(as.numeric(ENTConsFactorsUCon$getText()), warning = function(W) ERRConsUCon["visible"] <- TRUE)
		if (is.numeric(text) == TRUE & is.na(text) == FALSE) {
			ERRConsUCon["visible"] <- FALSE
			SETAnalysisOpt$ConsParam["conversion"] <- text
		} else ERRConsUCon["visible"] <- TRUE
	} else {
		ERRConsUCon["visible"] <- FALSE
	}
}))

####
#### Signal: Alternative Y-Label for consumption
################################################
gSignalConnect(ENTConsAxis, "activate", quote({
	text <- ENTConsAxis$getText()
	SETAnalysisOpt$ConsCharacters["axis"] <- text
}))

####
#### Signal: Selection for IC50 changed
#######################################
SelectionHandler <- gSignalConnect(COMIC50Selection, "changed", quote({
	if (COMIC50Selection$getActive() != -1) {
		SETAnalysisOpt$IC50 <- names(ListDataSets[which(ListDataSets == COMIC50Selection$getActiveText())])
	}
	#print(SETAnalysisOpt$IC50)
}))

####
#### Signal: Time Start for IC50 Calculation
############################################
ERRIC50TimeframeStart1 <- gtkLabel("-- Please enter a number for start of IC50 calculation! --")
	ERRIC50TimeframeStart1["visible"] <- FALSE
	BXErrormessages$packStart(ERRIC50TimeframeStart1, expand = FALSE, fill = FALSE)
ERRIC50TimeframeStart2 <- gtkLabel("-- The given timepoint for start of IC50 calculation can't be found in imported data! --")
	ERRIC50TimeframeStart2["visible"] <- FALSE
	BXErrormessages$packStart(ERRIC50TimeframeStart2, expand = FALSE, fill = FALSE)
ERRIC50Timeframe <- gtkLabel("-- Starting time point of IC50 calculation has to be before end point! --")
	ERRIC50Timeframe["visible"] <- FALSE
	BXErrormessages$packStart(ERRIC50Timeframe, expand = FALSE, fill = FALSE)

gSignalConnect(ENTIC50TimeStart, "activate", quote({
	text <- as.numeric(ENTIC50TimeStart$getText())
	if (SETAnalysisOpt$IC50calc != FALSE) {
		if (is.na(text) == TRUE) { #no number entered
			ERRIC50TimeframeStart1["visible"] <- TRUE
			ERRIC50TimeframeStart2["visible"] <- FALSE
			ERRIC50Timeframe["visible"] <- FALSE
		} else if (Data_input[1,1] != "empty") {
			#Data is already imported and a number is entered -> check whether it can be found in data
			for (i in 1:length(Data_input$X)) {
				if (Data_input$X[i] == text) { #number can be found
					SETAnalysisOpt$IC50Time[1] <- text
					SETAnalysisOpt$IC50Time[2] <- i
					ERRIC50TimeframeStart1["visible"] <- FALSE
					ERRIC50TimeframeStart2["visible"] <- FALSE
					if (SETAnalysisOpt$IC50Time[1] > SETAnalysisOpt$IC50Time[3]) ERRIC50Timeframe["visible"] <- TRUE
					else ERRIC50Timeframe["visible"] <- FALSE
					break
				}
				ERRIC50TimeframeStart1["visible"] <- FALSE
				ERRIC50TimeframeStart2["visible"] <- TRUE
				ERRIC50Timeframe["visible"] <- FALSE
			}
		} else { #no data imported yet
			SETAnalysisOpt$IC50Time[1] <- text
			ERRIC50TimeframeStart1["visible"] <- FALSE
			ERRIC50TimeframeStart2["visible"] <- FALSE
			ERRIC50Timeframe["visible"] <- FALSE
		}
	} else {
		ERRIC50TimeframeStart1["visible"] <- FALSE
		ERRIC50TimeframeStart2["visible"] <- FALSE
		ERRIC50Timeframe["visible"] <- FALSE
	}
	#print(SETAnalysisOpt$IC50Time)
}))

####
#### Signal: Time End for IC50 Calculation
##########################################
ERRIC50TimeframeEnd1 <- gtkLabel("-- Please enter a number for end of IC50 calculation! --")
	ERRIC50TimeframeEnd1["visible"] <- FALSE
	BXErrormessages$packStart(ERRIC50TimeframeEnd1, expand = FALSE, fill = FALSE)
ERRIC50TimeframeEnd2 <- gtkLabel("-- The given timepoint for end of IC50 calculation can't be found in imported data! --")
	ERRIC50TimeframeEnd2["visible"] <- FALSE
	BXErrormessages$packStart(ERRIC50TimeframeEnd2, expand = FALSE, fill = FALSE)

gSignalConnect(ENTIC50TimeEnd, "activate", quote({
	text <- as.numeric(ENTIC50TimeEnd$getText())
	if (SETAnalysisOpt$IC50calc != FALSE) {
		if (is.na(text) == TRUE) { #no number entered
			ERRIC50TimeframeEnd1["visible"] <- TRUE
			ERRIC50TimeframeEnd2["visible"] <- FALSE
			ERRIC50Timeframe["visible"] <- FALSE
		} else if (Data_input[1,1] != "empty") {
			#Data is already imported and a number is entered -> check whether it can be found in data
			for (i in 1:length(Data_input$X)) {
				if (Data_input$X[i] == text) { #number can be found
					SETAnalysisOpt$IC50Time[3] <- text
					SETAnalysisOpt$IC50Time[4] <- i
					ERRIC50TimeframeEnd1["visible"] <- FALSE
					ERRIC50TimeframeEnd2["visible"] <- FALSE
					if (SETAnalysisOpt$IC50Time[1] > SETAnalysisOpt$IC50Time[3]) ERRIC50Timeframe["visible"] <- TRUE
					else ERRIC50Timeframe["visible"] <- FALSE
					break
				}
				ERRIC50TimeframeEnd1["visible"] <- FALSE
				ERRIC50TimeframeEnd2["visible"] <- TRUE
				ERRIC50Timeframe["visible"] <- FALSE
			}
		} else { #no data imported yet
			SETAnalysisOpt$IC50Time[3] <- text
			ERRIC50TimeframeEnd1["visible"] <- FALSE
			ERRIC50TimeframeEnd2["visible"] <- FALSE
			ERRIC50Timeframe["visible"] <- FALSE
		}
	} else {
		ERRIC50TimeframeEnd1["visible"] <- FALSE
		ERRIC50TimeframeEnd2["visible"] <- FALSE
		ERRIC50Timeframe["visible"] <- FALSE
	}
	#print(SETAnalysisOpt$IC50Time)
}))

####
#### Signal: Skip for IC50
##########################
ERRIC50Skip <- gtkLabel("-- Please enter a positive number for skipping points in IC50 calculation! --")
	ERRIC50Skip["visible"] <- FALSE
	BXErrormessages$packStart(ERRIC50Skip, expand = FALSE, fill = FALSE)

gSignalConnect(ENTIC50Skip, "activate", quote({
	if (SETAnalysisOpt$IC50calc != FALSE) {
		text <- as.numeric(ENTIC50Skip$getText())
		if (is.na(text) == TRUE | text <= 0) {
			ERRIC50Skip["visible"] <- TRUE
		} else {
			SETAnalysisOpt$IC50Time[5] <- text
			ERRIC50Skip["visible"] <- FALSE
		}
	} else {
		ERRIC50Skip["visible"] <- FALSE
	}
	#print(SETAnalysisOpt$IC50Time)
}))

####
#### Signal: Model selection
############################
gSignalConnect(COMIC50Model, "changed", quote({
	if (COMIC50Model$getActiveText() == "2 parameter log-logistic function") {
		SETAnalysisOpt$IC50model <- "LL.2"
	} else if (COMIC50Model$getActiveText() == "3 parameter log-logistic function") {
		SETAnalysisOpt$IC50model <- "LL.3"
	} else if (COMIC50Model$getActiveText() == "4 parameter log-logistic function") {
		SETAnalysisOpt$IC50model <- "LL.4"
	} else if (COMIC50Model$getActiveText() == "5 parameter log-logistic function") {
		SETAnalysisOpt$IC50model <- "LL.5"
	}
	#print(SETAnalysisOpt$IC50model)
}))

####
#### Signal: Axis label for IC50 calculation
############################################
gSignalConnect(ENTIC50Axis, "activate", quote({
	SETAnalysisOpt$IC50Axis <- ENTIC50Axis$getText()
}))

####
#### Function: Check doses for IC50
###################################
ERRIC50Doses <- gtkLabel("-- Please enter numerical values > 0 or -1 only for the IC50 doses! --")
ERRIC50Doses["visible"] <- FALSE
	BXErrormessages$packStart(ERRIC50Doses, expand = FALSE, fill = FALSE)

FCheckIC50Doses <- function(T) {
	names <- levels(as.factor(T$Name))
	doses <- c()
	rem <- c()
	proceed <- 1
	if (length(TEMPDoses) != 0 && length(names) != 0) {
		for(i in 1:length(names)) {
			if (class(TEMPDoses[[names[i]]]$entry)[1] == "GtkEntry") {
				dose <- tryCatch(as.numeric(gtkEntryGetText(TEMPDoses[[names[i]]]$entry)), warning = function(W) {return(-2)})
				if (dose == -1 | dose >= 0) {
					doses <- c(doses, as.numeric(dose))
					ERRIC50Doses["visible"] <- FALSE
				} else {
					ERRIC50Doses["visible"] <- TRUE
					proceed <- 0
					break
				}
			}
			else rem <- c(rem, i)
		}
	}
	if (proceed == 1 & length(rem) != 0) SETAnalysisOpt$IC50doses <<- data.frame(Name = names[-rem], Dose = doses)
	else if (proceed == 1 & length(rem) == 0) SETAnalysisOpt$IC50doses <<- data.frame(Name = names, Dose = doses)
}

####
#### Signal: Calibrate Oxygen Consumption
#########################################
btnCalibrationImaginary <- gtkButton("Open Calibration")
gSignalConnect(btnSwitchCalibration, "clicked", quote({
	POPCalibrationExpl <- gtkMessageDialog(parent = mainApplication, "destroy-with-parent", type="question", button="none", "The calibration needs a dataset of timeresolved oxygen concentrations and its slope. If you didn't run this analysis yet, please do this before you continue.")
	POPCalibrationExpl$addButton("Go back to run required analysis", 1)
	POPCalibrationExpl$addButton("Continue with the calibration", 2)
	response <- POPCalibrationExpl$run()
	if (response == 2) {
		POPCalibrationExpl$destroy()
		gSignalEmit(btnCalibrationImaginary, "clicked")
	} else POPCalibrationExpl$destroy()
}))

####
#### Signal: Continue with Calibration
######################################
gSignalConnect(btnCalibrationImaginary, "clicked", quote({
	POPCalibrationExe <- gtkDialogNewWithButtons(title = "Calibration of oxygen accessibility", "destroy-with-parent", parent=mainApplication,"gtk-cancel", GtkResponseType["reject"])
	
	FCalBuildPopupLayout()
	POPCalibrationExe[["vbox"]]$add(BXCalPopup)
	POPCalibrationExe$addButton("Refresh data preview", 1)
	POPCalibrationExe$addButton("Run calibration", 2)
	POPCalibrationExe$addButton("Save calibration", 3)
	POPCalibrationExe$addButton("Save plot", 4)
	
	FCalUpdateDataPreview()
	
	action <- TRUE
	while (action == TRUE) {
		response <- POPCalibrationExe$run()
		if (response == 1) {
			FCalUpdateDataPreview()
		} else if (response == 2) {
			if (FCalRun() == FALSE) {
				LBLCalMessage$setLabel("The calibration could not be done, due to missing data sets or parameters. Please fill in all fields correctly and try again.")
			} else {
				FRCalResultsData["visible"] <- TRUE
				FRCalResultsPlot["visible"] <- TRUE
				LBLCalMessage$setLabel("The calibration was run successfully.")
				NBCal$setCurrentPage(2)
				FUpdateResultsPlot()
				FUpdateResultsData()
			}
		} else if (response == 3) {
			save <- tryCatch(get("CalibrationResults"), error = function(W) return(FALSE))
			if (class(save) != "list") LBLCalMessage$setLabel("There is no calibration result to be saved.")
			else {
				con <- file(paste(SETDataOutput$folder, "CalibrationResults.txt", sep="/"), open = "w")
				serialize(CalibrationResults, connection = con)
				close(con)
				LBLCalMessage$setLabel(paste0("The calibration data was saved to: ", paste(getwd(), SETDataOutput$folder, "CalibrationResults.txt", sep="/")))
			}
		} else if (response == 4) {
			save <- tryCatch(get("CalibrationResults"), error = function(W) return(FALSE))
			if (class(save) != "list") LBLCalMessage$setLabel("There is no calibration result to be saved as plot.")
			else {
				FSaveCalPlot()
				LBLCalMessage$setLabel(paste0("The calibration plot was saved to: ", paste(getwd(), SETDataOutput$folder, "CalibrationPlot.png", sep="/")))
			}
		} else {
			action <- FALSE
			POPCalibrationExe$destroy()
		}
		Sys.sleep(.1)
	}
}))

####
#### Function: Build Popup layout:
##################################
FCalBuildPopupLayout <- function() {
	
	## General Popup Layout
	BXCalPopup <<- gtkHBox(FALSE, 5)
	BXCalPopup$setSizeRequest(1000, 600)
	BXCalLeft <<- gtkVBox(FALSE, 5)
	BXCalRight <<- gtkVBox(FALSE, 5)
		BXCalPopup$packStart(BXCalLeft, expand = FALSE, fill = TRUE)
		BXCalPopup$packStart(BXCalRight, expand = TRUE, fill = TRUE)
	
	## Settings
	BXCalDataselection <<- gtkHBox(FALSE, 5)
	BXCalTimeselection1 <<- gtkHBox(FALSE, 5)
	BXCalTimeselection2 <<- gtkHBox(FALSE, 5)
	BXCalOxselection1 <<- gtkHBox(FALSE, 5)
	BXCalOxselection2 <<- gtkHBox(FALSE, 5)
	BXCalOxsat1 <<- gtkHBox(FALSE, 5)
	BXCalOxsat2 <<- gtkHBox(FALSE, 5)
	BXCalNames <<- gtkVBox(FALSE, 5)
	LBLCalMessage <<- gtkLabel("")
	LBLCalMessage["wrap"] <<- TRUE
		BXCalLeft$packStart(BXCalDataselection, expand = FALSE, fill = FALSE)
		BXCalLeft$packStart(BXCalTimeselection1, expand = FALSE, fill = FALSE)
		BXCalLeft$packStart(BXCalTimeselection2, expand = FALSE, fill = FALSE)
		BXCalLeft$packStart(BXCalOxselection1, expand = FALSE, fill = FALSE)
		BXCalLeft$packStart(BXCalOxselection2, expand = FALSE, fill = FALSE)
		BXCalLeft$packStart(BXCalOxsat1, expand = FALSE, fill = FALSE)
		BXCalLeft$packStart(BXCalOxsat2, expand = FALSE, fill = FALSE)
		BXCalLeft$packStart(BXCalNames, expand = FALSE, fill = FALSE)
		BXCalLeft$packStart(LBLCalMessage, expand = FALSE, fill = FALSE)

	## Dataselection
	LBLCalDataselection1 <<- gtkLabel("Use the oxygen values and their slope from the")
	LBLCalDataselection2 <<- gtkLabel("data.")
	COMCalDataselection <<- gtkComboBoxNewText()
	suitabledata <<- FALSE
	if (SETAnalysisOpt$slopeSelection["Data_raw"] == TRUE & TEMPChecksums$Data_raw[2] == TEMPChecksums$Data_rawSlope[1]) {
		COMCalDataselection$appendText(ListDataSets["Data_raw"])
		suitabledata <<- TRUE
	}
	if (SETAnalysisOpt$slopeSelection["Data_oxoDish"] == TRUE & TEMPChecksums$Data_oxoDish[2] == TEMPChecksums$Data_oxoDishSlope[1]) {
		COMCalDataselection$appendText(ListDataSets["Data_oxoDish"])
		suitabledata <<- TRUE
	}
	if (SETAnalysisOpt$slopeSelection["Data_oxoPlate"] == TRUE & TEMPChecksums$Data_oxoPlate[2] == TEMPChecksums$Data_oxoPlateSlope[1]) {
		COMCalDataselection$appendText(ListDataSets["Data_oxoPlate"])
		suitabledata <<- TRUE
	}
	if (SETAnalysisOpt$slopeSelection["Data_medium"] == TRUE & TEMPChecksums$Data_medium[2] == TEMPChecksums$Data_mediumSlope[1]) {
		COMCalDataselection$appendText(ListDataSets["Data_medium"])
		suitabledata <<- TRUE
	}
	if (suitabledata == FALSE) COMCalDataselection$appendText("no suitable")
	COMCalDataselection$setActive(0)
		BXCalDataselection$packStart(LBLCalDataselection1, expand = FALSE, fill = FALSE)
		BXCalDataselection$packStart(COMCalDataselection, expand = FALSE, fill = FALSE)
		BXCalDataselection$packStart(LBLCalDataselection2, expand = FALSE, fill = FALSE)

	## Timeselection
	LBLCalTimeselection1 <<- gtkLabel("The hill to be analysed lies approximately in the timeframe from")
	LBLCalTimeselection2 <<- gtkLabel("to")
	LBLCalTimeselection3 <<- gtkLabel("in the given output unit.")
	ENTCalTStart <<- gtkEntry()
	ENTCalTStart$setSizeRequest(70,20)
	ENTCalTStart$setText("0")
	ENTCalTStop <<- gtkEntry()
	ENTCalTStop$setSizeRequest(70,20)
	ENTCalTStop$setText("3")
		BXCalTimeselection1$packStart(LBLCalTimeselection1, expand = FALSE, fill = FALSE)
		BXCalTimeselection2$packStart(ENTCalTStart, expand = FALSE, fill = FALSE)
		BXCalTimeselection2$packStart(LBLCalTimeselection2, expand = FALSE, fill = FALSE)
		BXCalTimeselection2$packStart(ENTCalTStop, expand = FALSE, fill = FALSE)
		BXCalTimeselection2$packStart(LBLCalTimeselection3, expand = FALSE, fill = FALSE)

	## Oxselection
	LBLCalOxselection1 <<- gtkLabel("Use data from")
	LBLCalOxselection2 <<- gtkLabel("to")
	LBLCalOxselection3 <<- gtkLabel("oxygen concentration for the linear fit of the kla-value.")
	ENTCalOxStart <<- gtkEntry()
	ENTCalOxStart$setSizeRequest(70,20)
	ENTCalOxStart$setText("40")
	ENTCalOxStop <<- gtkEntry()
	ENTCalOxStop$setSizeRequest(70,20)
	ENTCalOxStop$setText("90")
		BXCalOxselection1$packStart(LBLCalOxselection1, expand = FALSE, fill = FALSE)
		BXCalOxselection1$packStart(ENTCalOxStart, expand = FALSE, fill = FALSE)
		BXCalOxselection1$packStart(LBLCalOxselection2, expand = FALSE, fill = FALSE)
		BXCalOxselection1$packStart(ENTCalOxStop, expand = FALSE, fill = FALSE)
		BXCalOxselection2$packStart(LBLCalOxselection3, expand = FALSE, fill = FALSE)

	## Saturated oxygen value
	LBLCalOxsat1 <<- gtkLabel("The fully saturated oxygen concentration is")
	LBLCalOxsat2 <<- gtkLabel(".")
	ENTCalOxsat <<- gtkEntry()
	ENTCalOxsat$setSizeRequest(70,20)
	ENTCalOxsat$setText("95")
		BXCalOxsat1$packStart(LBLCalOxsat1, expand = FALSE, fill = FALSE)
		BXCalOxsat1$packStart(ENTCalOxsat, expand = FALSE, fill = FALSE)
		BXCalOxsat1$packStart(LBLCalOxsat2, expand = FALSE, fill = FALSE)
	
	## Calibration names
	LBLCalNames <<- gtkLabel("Please enter the names for the different calibrations (one per number in the template):")
		BXCalNames$packStart(LBLCalNames, expand = FALSE, fill = FALSE)
	ListNamesLayouts <<- list()
	numbers <- as.character(levels(as.factor(Template$Number)))
	for(i in 1:length(numbers)) {
		if (numbers[i] != 0) {
			ListNamesLayouts[[numbers[i]]] <<- list()
			ListNamesLayouts[[numbers[i]]][["box"]] <<- gtkHBox(FALSE, 5)
			ListNamesLayouts[[numbers[i]]][["label"]] <<- gtkLabel(paste0(numbers[i], ":"))
			ListNamesLayouts[[numbers[i]]][["entry"]] <<- gtkEntry()
			ListNamesLayouts[[numbers[i]]][["entry"]]$setSizeRequest(70, 20)
			ListNamesLayouts[[numbers[i]]][["entry"]]$setText(Template$Name[which(Template$Number == numbers[i])[1]])
			ListNamesLayouts[[numbers[i]]][["box"]]$packStart(ListNamesLayouts[[numbers[i]]][["label"]], expand = FALSE, fill = FALSE)
			ListNamesLayouts[[numbers[i]]][["box"]]$packStart(ListNamesLayouts[[numbers[i]]][["entry"]], expand = FALSE, fill = FALSE)
				BXCalNames$packStart(ListNamesLayouts[[numbers[i]]][["box"]], expand = FALSE, fill = FALSE)
		}
	}
	
	## Data preview Frame
	FRCalPreviewData <<- gtkFrame()
	LBLCalPreviewData <<- gtkLabel("Data preview")
	
	## Results Data Frame
	FRCalResultsData <<- gtkFrame()
	LBLCalResultsData <<- gtkLabel("Resulting values")
	BXCalResultsData <<- gtkVBox(FALSE, 5)
	FRCalResultsData$add(BXCalResultsData)
	
	## Results Plot Frame
	FRCalResultsPlot <<- gtkFrame()
	LBLCalResultsPlot <<- gtkLabel("Plot result")
	
	## Notebook for display:
	NBCal <<- gtkNotebook()
		BXCalRight$packStart(NBCal, expand = TRUE, fill = TRUE)
	NBCal$appendPage(FRCalPreviewData, tab.label = LBLCalPreviewData)
	NBCal$appendPage(FRCalResultsData, tab.label = LBLCalResultsData)
		FRCalResultsData["visible"] <<- FALSE
	NBCal$appendPage(FRCalResultsPlot, tab.label = LBLCalResultsPlot)
		FRCalResultsPlot["visible"] <<- FALSE
}

####
#### Function: Update Data preview:
###################################
FCalUpdateDataPreview <- function() {
	
	Data <- tryCatch(get(names(ListDataSets)[which(ListDataSets == COMCalDataselection$getActiveText())]), error = function(W) return(data.frame()))
	
	if (length(Data[1,1]) == 0) return()
	
	tryCatch(DACalPreviewData$destroy(), error = function(W) return())
	DACalPreviewData <<- gtkDrawingArea(show = TRUE)
	FRCalPreviewData$add(DACalPreviewData)
	DACalPreviewData$setSizeRequest(100,100)
	asCairoDevice(DACalPreviewData, pointsize = 12)#10
	Sys.sleep(.1)
	
	par(mar = c(5,5,5,2))
	plot(0,0,ylim = c(min(Data[,2]), max(Data[,2])), xlim = c(min(Data[,1]), max(Data[,1])), xlab = "Time", ylab = "Oxygen level", main = "Data preview")
  grid(col = "grey")
	for(i in 2:dim(Data)[2]) {
		lines(Data[,1], Data[,i], col = "black")
	}
	abline(h = switch(as.character(tryCatch(as.numeric(ENTCalOxStart$getText()), warning = function(W) return())), "NA" = min(Data[,2]), as.numeric(ENTCalOxStart$getText())), col = "red")
	abline(h = switch(as.character(tryCatch(as.numeric(ENTCalOxStop$getText()), warning = function(W) return())), "NA" = max(Data[,2]), as.numeric(ENTCalOxStop$getText())), col = "red")
	abline(v = switch(as.character(tryCatch(as.numeric(ENTCalTStart$getText()), warning = function(W) return())), "NA" = min(Data[,1]), as.numeric(ENTCalTStart$getText())), col = "red")
	abline(v = switch(as.character(tryCatch(as.numeric(ENTCalTStop$getText()), warning = function(W) return())), "NA" = max(Data[,1]), as.numeric(ENTCalTStop$getText())), col = "red")
}

####
#### Function: Check Calibration input and start Analysis
#########################################################
FCalRun <- function() {
	## get Data
	Data <- tryCatch(get(names(ListDataSets)[which(ListDataSets == COMCalDataselection$getActiveText())]), error = function(W) return(data.frame()))
	if (length(Data[1,1]) == 0) return(FALSE)
	DataSlope <- tryCatch(get(paste0(names(ListDataSets)[which(ListDataSets == COMCalDataselection$getActiveText())], "Slope")), error = function(W) return(data.frame()))
	if (length(DataSlope[1,1]) == 0) return(FALSE)
	
	## Check numerical values
	numeric <- c(ENTCalOxStart, ENTCalOxStop, ENTCalTStart, ENTCalTStop, ENTCalOxsat)
	CalibrationNumbers <- c()
	for (i in 1:length(numeric)) {
		text <- tryCatch(as.numeric(gtkEntryGetText(numeric[[i]])), warning = function(W) return(NA))
		if (is.na(text) == TRUE) return(FALSE)
		else CalibrationNumbers[i] <- text
	}
	if (CalibrationNumbers[1] >= CalibrationNumbers[2]) return(FALSE)
	if (CalibrationNumbers[3] >= CalibrationNumbers[4]) return(FALSE)
	
	FCalculateCalibration(Data, DataSlope, CalibrationNumbers)
	return(TRUE)
}

####
#### Function: Update the calibration results plot
##################################################
FUpdateResultsPlot <- function() {
	if (length(CalibrationResults) == 0) return(FALSE)
	
	tryCatch(DACalResultsPlot$destroy(), error = function(W) return())
	DACalResultsPlot <<- gtkDrawingArea(show = TRUE)
	FRCalResultsPlot$add(DACalResultsPlot)
	DACalResultsPlot$setSizeRequest(100,100)
	asCairoDevice(DACalResultsPlot, pointsize = 12)
	Sys.sleep(.1)
	
	topylim <- 0
	for (i in 1:length(CalibrationResults)) {
		if (length(CalibrationResults[[i]]$prediction[,1]) != 0 && max(CalibrationResults[[i]]$prediction[,1]) >= topylim) topylim <- max(CalibrationResults[[i]]$prediction[,1])
	}
	par(mar = c(5,5,5,2))
	plot(0,0,col = "white", xlim = c(0,100), ylim = c(-50, as.integer(topylim)), xlab = "Oxygen level", ylab = "Diffusion flux", main = "Calibration lines oxygen consumption")
	grid(col = "grey")
	
	plotcolours <- c()
	names <- c()
	testdata <- seq(0, 100, length = 301)
	for (i in 1:length(CalibrationResults)) {
		if (i%%8 == 1) colour <- "black"
		if (i%%8 == 2) colour <- "red"
		if (i%%8 == 3) colour <- "orange"
		if (i%%8 == 4) colour <- "wheat"
		if (i%%8 == 5) colour <- "lightgrey"
		if (i%%8 == 6) colour <- "blue"
		if (i%%8 == 7) colour <- "magenta"
		if (i%%8 == 0) colour <- "blue"
		plotcolours <- c(plotcolours, colour)
		names <- c(names, CalibrationResults[[i]]$name)
		points(CalibrationResults[[i]]$fitdata$Ox, CalibrationResults[[i]]$fitdata$Flux, col = colour)
		if (length(CalibrationResults[[i]]$prediction[,1]) != 0) lines(testdata, CalibrationResults[[i]]$prediction[,1], col = colour, lwd = 2)
	}
	legend("topright", legend = names, col = plotcolours, lty = "solid", lwd = "2", bg = "white", seg.len=1, cex = 0.8)
}

####
#### Function: Save the calibration results plot
################################################
FSaveCalPlot <- function(inwidth = 9, inheight = 7, inpt = 12, infilename = "CalibrationPlot.png") {
	if (length(CalibrationResults) == 0) return(FALSE)
	
	Cairo(width = inwidth, height = inheight, pointsize = inpt, filename = paste(SETDataOutput$folder, infilename, sep="/"))
	Sys.sleep(.1)
	
	par(mar = c(5,5,5,2))
	plot(0,0,col = "white", xlim = c(0,100), ylim = c(-50, 500), xlab = "Oxygen level", ylab = "Diffusion flux", main = "Calibration lines oxygen consumption")
	grid(col = "grey")
	
	plotcolours <- c()
	names <- c()
	testdata <- seq(0, 100, length = 301)
	for (i in 1:length(CalibrationResults)) {
		if (i%%8 == 1) colour <- "black"
		if (i%%8 == 2) colour <- "red"
		if (i%%8 == 3) colour <- "orange"
		if (i%%8 == 4) colour <- "wheat"
		if (i%%8 == 5) colour <- "lightgrey"
		if (i%%8 == 6) colour <- "blue"
		if (i%%8 == 7) colour <- "magenta"
		if (i%%8 == 0) colour <- "blue"
		plotcolours <- c(plotcolours, colour)
		names <- c(names, CalibrationResults[[i]]$name)
		points(CalibrationResults[[i]]$fitdata$Ox, CalibrationResults[[i]]$fitdata$Flux, col = colour)
		lines(testdata, CalibrationResults[[i]]$prediction[,1], col = colour, lwd = 2)
	}
	legend("topright", legend = names, col = plotcolours, lty = "solid", lwd = "2", bg = "white", seg.len=1, cex = 0.8)
	
	Sys.sleep(.1)
	dev.off()
}

####
#### Function: Update calibration results display
#################################################
FUpdateResultsData <- function() {
	if (length(CalibrationResults) == 0) return(FALSE)
	
	numbers <- as.character(levels(as.factor(Template$Number)))
	children <- BXCalResultsData$getChildren()
	if (length(children) > 0) {
		for (i in 1:length(children)) children[[i]]$destroy()
	}
	for(i in 1:length(numbers)) {
		if (class(CalibrationResults[[numbers[i]]]) == "list" && dim(summary(CalibrationResults[[numbers[i]]]$nlsResult)$coefficients)[1] != 0) {
			summary <- summary(CalibrationResults[[numbers[i]]]$nlsResult)
			ListNamesLayouts[[numbers[i]]][["name"]] <<- gtkLabel(paste0(numbers[i], ": ", CalibrationResults[[numbers[i]]]$name))
			ListNamesLayouts[[numbers[i]]][["res"]] <<- gtkHBox(FALSE, 5)
			ListNamesLayouts[[numbers[i]]][["reskla"]] <<- gtkLabel(paste("kLa", format(summary$coefficients[1,1]), sep = "\n"))
			ListNamesLayouts[[numbers[i]]][["reserr"]] <<- gtkLabel(paste("Std. Error", format(summary$coefficients[1,2]), sep = "\n"))
			ListNamesLayouts[[numbers[i]]][["restval"]] <<- gtkLabel(paste("t value", format(summary$coefficients[1,3]), sep = "\n"))
			ListNamesLayouts[[numbers[i]]][["respval"]] <<- gtkLabel(paste("Pr(>|t|)", format(summary$coefficients[1,4]), sep = "\n"))
			ListNamesLayouts[[numbers[i]]][["resR"]] <<- gtkLabel(paste("R squared", format(summary$adj.r.squared), sep = "\n"))
			ListNamesLayouts[[numbers[i]]][["res"]]$packStart(ListNamesLayouts[[numbers[i]]][["reskla"]], expand = TRUE, fill = FALSE)
			ListNamesLayouts[[numbers[i]]][["res"]]$packStart(ListNamesLayouts[[numbers[i]]][["reserr"]], expand = TRUE, fill = FALSE)
			ListNamesLayouts[[numbers[i]]][["res"]]$packStart(ListNamesLayouts[[numbers[i]]][["restval"]], expand = TRUE, fill = FALSE)
			ListNamesLayouts[[numbers[i]]][["res"]]$packStart(ListNamesLayouts[[numbers[i]]][["respval"]], expand = TRUE, fill = FALSE)
			ListNamesLayouts[[numbers[i]]][["res"]]$packStart(ListNamesLayouts[[numbers[i]]][["resR"]], expand = TRUE, fill = FALSE)
			BXCalResultsData$packStart(ListNamesLayouts[[numbers[i]]][["name"]], expand = FALSE, fill = FALSE)
			BXCalResultsData$packStart(ListNamesLayouts[[numbers[i]]][["res"]], expand = FALSE, fill = FALSE)
		}
	}
}

#########################################################################################
#### Analysis Options execute
#########################################################################################

#### Create vector containing all elements to be checked before execution
CheckInputAnalysisOpt <- c(ENTSlopeSmooth, ENTSlopePoints, CKBSelectionRawAv, CKBSelectionMedium, CKBSelectionMediumAv, CKBSelectionSmooth, CKBSelectionSlope, CKBSelectionCons, CKBSelectionIC50, ENTMediumCalName, ENTMediumCalTarget, btnAverageName, ENTConsAxis, ENTConsSat, COMIC50Selection, ENTIC50TimeStart, ENTIC50TimeEnd, ENTIC50Skip, COMIC50Model, ENTIC50Axis, COMFormatSelectData, ENTFormatCutXStart, ENTFormatCutXEnd, ENTFormatTransformXm, ENTFormatTransformXb, ENTFormatTransformY, btnDataLayoutTimesec, btnPlotAxesH1sec)

####
#### Function: Activate all Input for Analysis Options
######################################################
FCheckAnalysisOpt <- function(Input) {
	for (i in 1:length(Input)) {
		if (class(Input[[i]])[1] == "GtkRadioButton") {
			gSignalEmit(Input[[i]], "toggled")
		} else if (class(Input[[i]])[1] == "GtkEntry") {
			gSignalEmit(Input[[i]], "activate")
		} else if (class(Input[[i]])[1] == "GtkCheckButton") {
			gSignalEmit(Input[[i]], "toggled")
		} else if (class(Input[[i]])[1] == "GtkComboBox") {
			gSignalEmit(Input[[i]], "changed")
		}
	}
	
	## Check Doses for IC50
	FCheckIC50Doses(Template)
}

####
#### Function: Refresh IC50 Selection Combo Box
###############################################
TEMPIC50Selection <- c(	"Data_raw" = TRUE,
						"Data_rawSm" = FALSE,
						"Data_rawCons" = FALSE,
						"Data_rawAv" = FALSE,
						"Data_rawAvSm" = FALSE,
						"Data_rawAvCons" = FALSE,
						"Data_oxoDish" = FALSE,
						"Data_oxoDishSm" = FALSE,
						"Data_oxoDishCons" = FALSE,
						"Data_oxoDishAv" = FALSE,
						"Data_oxoDishAvSm" = FALSE,
						"Data_oxoDishAvCons" = FALSE,
						"Data_oxoPlate" = FALSE,
						"Data_oxoPlateSm" = FALSE,
						"Data_oxoPlateCons" = FALSE,
						"Data_oxoPlateAv" = FALSE,
						"Data_oxoPlateAvSm" = FALSE,
						"Data_oxoPlateAvCons" = FALSE,
						"Data_hydroPlate" = FALSE,
						"Data_hydroPlateSm" = FALSE,
						"Data_hydroPlateAv" = FALSE,
						"Data_hydroPlateAvSm" = FALSE,
						"Data_medium" = FALSE,
						"Data_mediumSm" = FALSE,
						"Data_mediumCons" = FALSE,
						"Data_mediumAv" = FALSE,
						"Data_mediumAvSm" = FALSE,
						"Data_mediumAvCons" = FALSE )

FRefreshIC50Selection <- function() {
	## save position of selected data set
	position <- COMIC50Selection$getActive()
	## remove all entries
	gSignalHandlerBlock(COMIC50Selection, SelectionHandler)
	COMIC50Selection$setActive(0)
	while(COMIC50Selection$getActive() == 0) {
		COMIC50Selection$removeText(0)
		COMIC50Selection$setActive(0)
	}
	## add all entries and set to saved position
	for (i in 1:length(TEMPIC50Selection)) {
		if (TEMPIC50Selection[i] == TRUE) COMIC50Selection$appendText(ListDataSets[names(TEMPIC50Selection)[i]])
	}
	gSignalHandlerUnblock(COMIC50Selection, SelectionHandler)
	if (position != -1) COMIC50Selection$setActive(position)
	else COMIC50Selection$setActive(0)
	if (COMIC50Selection$getActive() == -1) COMIC50Selection$setActive(0)
}

####
#### Function: Refresh Formatting Selection Combo Box
#####################################################
TEMPFormatSelectData <- c(	"Data_raw" = TRUE,
							"Data_oxoDish" = FALSE,
							"Data_oxoPlate" = FALSE,
							"Data_hydroPlate" = FALSE,
							"Data_medium" = FALSE )

FRefreshFormattingSelection <- function() {
	## save position of selected data set
	position <- COMFormatSelectData$getActive()
	## remove all entries
	gSignalHandlerBlock(COMFormatSelectData, SelectionHandlerFormatting)
	COMFormatSelectData$setActive(1)
	while (COMFormatSelectData$getActive() == 1) {
		COMFormatSelectData$removeText(1)
		COMFormatSelectData$setActive(1)
	}
	## add all entries and set to saved position
	for (i in 1:length(TEMPFormatSelectData)) {
		if (TEMPFormatSelectData[i] == TRUE) COMFormatSelectData$appendText(ListDataSets[names(TEMPFormatSelectData)[i]])
	}
	gSignalHandlerUnblock(COMFormatSelectData, SelectionHandlerFormatting)
	COMFormatSelectData$setActive(position)
	if (COMFormatSelectData$getActive() == -1) COMFormatSelectData$setActive(0)
}